import java.io.*;
import com.nttdocomo.system.*;

public class SplitterInputStream extends InputStream {

    private InputStream is;
    private boolean isGIF;
    private int subblockSize, count;

    private int[] mark_buf;
    private int p1 = -1, p2 = -1;

    public int read() throws IOException {
	if (p1 != -1) {
	    int c = mark_buf[p1++];
	    if (p1 == p2)
		p1 = p2 = -1;
	    return c;
	}

	for (;;) {
	    if (is == null) {
		try {
		    ImageStore istore = ImageStore.selectEntry();
		    is = istore.getInputStream();

		    int tmp = is.read();
		    if (tmp == 0xFF) { // JPEG
			isGIF = false;
			is.skip(0x277- 1);
		    }
		    else if (tmp == 0x47) { // GIF
			isGIF = true;
			is.skip(34 + 2 - 1);
			subblockSize = is.read();
			count = 0;
		    }
		    else {
			is.close();
			return -1;
		    }
		}
		catch (Exception e) {
		    return -1;
		}
	    }

	    if (isGIF && count == subblockSize) {
		subblockSize = is.read();
		count = 0;
		if (subblockSize == 0) {
		    is.close();
		    is = null;
		    return -1;
		}
	    }
	    count++;
	    
	    int c = is.read();
	    if (c != -1) {
		if (p2 != -1 && p2 < mark_buf.length)
		    mark_buf[p2++] = c;
		return c;
	    }

	    is.close();
	    is = null;
	}
    }

    public void close() throws IOException {
	if (is != null) {
	    is.close();
	    is = null;
	}
    }

    public void mark(int readLimit) {
	mark_buf = new int[readLimit];
	p2 = 0;
    }

    public void reset() {
	p1 = 0;
    }
}
