import java.io.*;
import java.util.*;

public class GBCore extends Thread {
    
    private final GBCanvas gui;
    
    public GBCore(GBCanvas _gui) {
	gui = _gui;
    }

    // hw
    private static final int HW_GB = 1;
    private static final int HW_SGB = 2;
    private static final int HW_GBC = 3;
    // save
    private byte[][] WRAM;
    private int[] HIRAM;
    private int[] snd_regs;
    private int R_P1;    // FF00 Read Joypad Info
    private int R_SB;    // FF01 Serial Transfer Data
    private int R_SC;    // FF02 Serial I/O Control
    private int R_DIV;   // FF04 Timer Divider
    private int R_TIMA;  // FF05 Timer Counter
    private int R_TMA;   // FF06 Timer Modulo
    private int R_TAC;   // FF07 Timer Control
    private int R_IF;    // FF0F Interrupt Flag
    private int R_LCDC;  // FF40 LCDC Control
    private int R_STAT;  // FF41 LCD Status
    private int R_SCY;   // FF42 Scroll Screen Y
    private int R_SCX;   // FF43 Scroll Screen X
    private int R_LY;    // FF44 LCDC Y-Coord
    private int R_LYC;   // FF45 LY Compare
                         // FF46 DMA Transfer
    private int R_BGP;   // FF47 BG Palette Data
    private int R_OBP0;  // FF48 Object Palette 0 Data
    private int R_OBP1;  // FF49 Object Palette 1 Data
    private int R_WY;    // FF4A Window Y pos
    private int R_WX;    // FF4B Window Y pos
    // GBC only
    private int R_KEY1;  // FF4D CPU Speed Select
    private int R_VBK;   // FF4F VRAM Bank Select
    private int R_HDMA1; // FF51 HBL General DMA
    private int R_HDMA2; // FF52 HBL General DMA
    private int R_HDMA3; // FF53 HBL General DMA
    private int R_HDMA4; // FF54 HBL General DMA
    private int R_HDMA5; // FF55 HBL General DMA
    private int R_RP;    // FF56 Infrared Comms
    private int R_BCPS;  // FF68 Background Color Index
                         // FF69 Background Color Data
    private int R_OCPS;  // FF6A Object Color Index
                         // FF6B Object Color Data
    private int R_SVBK;  // FF70 RAM Bank Select
    private int R_IE;    // FFFF Interrupt Enable
    // no save
    private int hw_type;

    private final void hw_init() {
	WRAM = new byte[8][0x1000];
	HIRAM = new int[0x80];
	snd_regs = new int[0x30];
    }

    private final void hw_reset() {
	R_SC = 0;
	R_DIV = 0;
	R_TIMA = 0;
	R_TMA = 0;
	R_TAC =0;
	R_STAT = 0;
	R_LCDC = 0x91;
	R_SCY = 0;
	R_SCX = 0;
	R_LY = 153;
	R_LYC = 0;
	R_BGP = 0xFC;
	R_OBP0 = 0xFF;
	R_OBP1 = 0xFF;
	R_WY = 0;
	R_WX = 0;
	R_IF = 0;
	R_IE = 0;
    }

    private final int hw_read(int addr) {
	int data;

	switch (addr) {
	case 0x00:
	    return R_P1;
	case 0x01:
	    return R_SB;
	case 0x02:
	    data = R_SC;
	    R_SC &= 0x7F;
	    return data;
	    //return (R_SC & 0x83) | 0x7C; // TGBdual
	case 0x04:
	    return R_DIV;
	case 0x05:
	    return R_TIMA;
	case 0x06:
	    return R_TMA;
	case 0x07:
	    return R_TAC;
	case 0x0F:
	    return R_IF;

	    /*
	      byte apu::read(word adr)
	      {
	      if (adr==0xff26)
	      return (!snd->stat.master_enable)?0x00:
	      (0x80|(((snd->stat.sq1_playing&&snd->stat.wav_vol)?1:0)|
	      ((snd->stat.sq2_playing&&snd->stat.wav_vol)?2:0)|
	      ((snd->stat.wav_enable&&snd->stat.wav_playing&&snd->stat.wav_vol)?4:0)|
	      ((snd->stat.noi_playing&&snd->stat.noi_vol)?8:0)));
	      else
	      return snd->mem[adr-0xff10];
	      }
	    */
	case 0x26: // NR52
	    return 0x80;

	case 0x10: // NR10
	case 0x11: // NR11
	case 0x12: // NR12
	case 0x13: // NR13
	case 0x14: // NR14
	case 0x15: // NR20??
	case 0x16: // NR21
	case 0x17: // NR22
	case 0x18: // NR23
	case 0x19: // NR24
	case 0x1A: // NR30
	case 0x1B: // NR31
	case 0x1C: // NR32
	case 0x1D: // NR33
	case 0x1E: // NR34
	case 0x1F: // NR40??
	case 0x20: // NR41
	case 0x21: // NR42
	case 0x22: // NR43
	case 0x23: // NR44
	case 0x24: // NR50
	case 0x25: // NR51
	    //case 0x26: // NR52
	case 0x30: // AUD3WAVERAM (16bytes)
	case 0x31:
	case 0x32:
	case 0x33:
	case 0x34:
	case 0x35:
	case 0x36:
	case 0x37:
	case 0x38:
	case 0x39:
	case 0x3A:
	case 0x3B:
	case 0x3C:
	case 0x3D:
	case 0x3E:
	case 0x3F:
	    return snd_regs[addr - 0x10];

	case 0x40:
	    return R_LCDC;
	case 0x41:
	    return R_STAT | 0x80; //TGBdual
	case 0x42:
	    return R_SCY;
	case 0x43:
	    return R_SCX;
	case 0x44:
	    return R_LY;
	case 0x045:
	    return R_LYC;
	case 0x46: // DMA
	    return 0;
	case 0x47:
	    return R_BGP;
	case 0x48:
	    return R_OBP0;
	case 0x49:
	    return R_OBP1;
	case 0x4A:
	    return R_WY;
	case 0x4B:
	    return R_WX;
	    
	    // GBC only
	case 0x4D: // KEY1
	    //return R_KEY1;
	    return (cpu_speed != 0 ? 0x80 : (R_KEY1 & 1) != 0 ? 1 : 0x7E);
	case 0x4F:
	    return R_VBK;
	case 0x51:
	    return R_HDMA1;
	case 0x52:
	    return R_HDMA2;
	case 0x53:
	    return R_HDMA3;
	case 0x54:
	    return R_HDMA4;
	case 0x55:
	    return R_HDMA5;
	case 0x56: // RP
	    return 0;
	case 0x68:
	    return R_BCPS;
	case 0x69: // BCPD
	    return palette_ram[R_BCPS & 0x3F];
	case 0x6A:
	    return R_OCPS;
	case 0x6B: // OCPD
	    return palette_ram[0x40 + (R_OCPS & 0x3F)];
	case 0x70:
	    return R_SVBK;

	case 0xFF: // IE
	    return R_IE;
	    
	default:
	    throw new Error("hw_read " + Integer.toHexString(addr));
	}
    }

    private final void hw_write(int addr, int data) {
	switch (addr) {
	case 0x00:
	    R_P1 = data;
	    pad_refresh();
	    return;
	case 0x01:
	    R_SB = data;
	    return;
	case 0x02:
	    if (hw_type >= HW_GBC) {
		R_SC = data & 0x83;
		//XXX
	    }
	    else {
		R_SC = data & 0x81;
		//XXX
	    }
	    return;
	case 0x04:
	    R_DIV = 0;
	    return;
	case 0x05:
	    R_TIMA = data;
	    return;
	case 0x06:
	    R_TMA = data;
	    return;
	case 0x07:
	    //XXX
	    R_TAC = data;
	    return;
	case 0x0F:
	    R_IF = data;
	    return;

	case 0x10: // NR10
	case 0x11: // NR11
	case 0x12: // NR12
	case 0x13: // NR13
	case 0x14: // NR14
	case 0x15: // NR20??
	case 0x16: // NR21
	case 0x17: // NR22
	case 0x18: // NR23
	case 0x19: // NR24
	case 0x1A: // NR30
	case 0x1B: // NR31
	case 0x1C: // NR32
	case 0x1D: // NR33
	case 0x1E: // NR34
	case 0x1F: // NR40??
	case 0x20: // NR41
	case 0x21: // NR42
	case 0x22: // NR43
	case 0x23: // NR44
	case 0x24: // NR50
	case 0x25: // NR51
	case 0x26: // NR52
	case 0x30: // AUD3WAVERAM (16bytes)
	case 0x31:
	case 0x32:
	case 0x33:
	case 0x34:
	case 0x35:
	case 0x36:
	case 0x37:
	case 0x38:
	case 0x39:
	case 0x3A:
	case 0x3B:
	case 0x3C:
	case 0x3D:
	case 0x3E:
	case 0x3F:
	    snd_regs[addr - 0x10] = data;
	    return;

	case 0x40:
	    if ((data & 0x80) != 0 && (R_LCDC & 0x80) == 0) { // LCD On/Off
		R_LY = 0;
		now_win_line = 0;
	    }
	    R_LCDC = data;
	    return;
	case 0x41:
	    /*
	      if (hw_type == HW_GB && (R_STAT & 2) == 0) // GB̃oO炵B
	      R_IF |= INT_LCDC;
	    */
	    R_STAT = (R_STAT & (STAT_COINCIDENCE|STAT_MODE)) | (data & 0x78);
	    return;
	case 0x42:
	    R_SCY = data;
	    return;
	case 0x43:
	    R_SCX = data;
	    return;
	case 0x44:
	    //R_LY = 0;
	    now_win_line = 0;
	    return;
	case 0x45:
	    R_LYC = data;
	    //XXX stat_trigger();
	    return;
	case 0x46: // DMA
	    hw_dma(data);
	    return;
	case 0x47:
	    R_BGP = data;
	    pal_write_dmg(0, 0, data);
	    //pal_write_dmg(8, 0, data);
	    return;
	case 0x48:
	    R_OBP0 = data;
	    pal_write_dmg(64, 1, data);
	    return;
	case 0x49:
	    R_OBP1 = data;
	    pal_write_dmg(72, 2, data);
	    return;
	case 0x4A:
	    R_WY = data;
	    return;
	case 0x4B:
	    R_WX = data;
	    return;

	    // GBC only
	case 0x4D:
	    R_KEY1 = (R_KEY1 & 0x80) | (data & 1);
	    return;
	case 0x4F:
	    R_VBK = data;
	    mem_set_VRAM_bank();
	    return;
	case 0x51:
	    R_HDMA1 = data;
	    return;
	case 0x52:
	    R_HDMA2 = data;
	    return;
	case 0x53:
	    R_HDMA3 = data;
	    return;
	case 0x54:
	    R_HDMA4 = data;
	    return;
	case 0x55: // HDMA5
	    hw_hdma(data);
	    return;
	case 0x56:
	    R_RP = data;
	    return;
	case 0x68:
	    R_BCPS = data;
	    return;
	case 0x69: // BCPD
	    lcd_set_palette(R_BCPS & 0x3F, data);
	    if ((R_BCPS & 0x80) != 0)
		R_BCPS = (R_BCPS + 1) & 0xBF;
	    return;
	case 0x6A:
	    R_OCPS = data;
	    return;
	case 0x6B: // OCPD
	    lcd_set_palette(0x40 + (R_OCPS & 0x3F), data);
	    if ((R_OCPS & 0x80) != 0)
		R_OCPS = (R_OCPS + 1) & 0xBF;
	    return;
	case 0x70:
	    R_SVBK = data;
	    mem_set_WRAM_bank();
	    return;

	case 0xFF:
	    R_IE = data;
	    return;

	default:
	    throw new Error("hw_write " + Integer.toHexString(addr) + " " + Integer.toHexString(data));
	}
    }

    private final void hw_dma(int data) {
	int addr = data << 8;
	for (int i = 0; i < 0xA0; i++)
	    OAM[i] = mem_read(addr + i);
    }

    private final void hw_hdma(int data) {
	if ((data & 0x80) != 0) { // HDMA
	    R_HDMA5 = data & 0x7F;
	    return;
	}

	// GDMA
	int src = (R_HDMA1 << 8) | (R_HDMA2 & 0xF0);
	int dst = 0x8000 | ((R_HDMA3 & 0x1F) << 8) | (R_HDMA4 & 0xF0);
	int cnt = (data + 1) << 4;
	//gdma_rest=456*2+cnt*2*(cpu_speed+1); // CPU p[
	while (cnt-- != 0)
	    mem_write(dst++, mem_read(src++));
	R_HDMA1 = (src >> 8) & 0xFF;
	R_HDMA2 = src & 0xF0;
	R_HDMA3 = (dst >> 8) & 0x1F;
	R_HDMA4 = dst & 0xF0;
	R_HDMA5 = 0xFF;
    }

    private final void hw_hdma() {
	if ((R_HDMA5 & 0x80) != 0)
	    return;

	int src = (R_HDMA1 << 8) | (R_HDMA2 & 0xF0);
	int dst = 0x8000 | ((R_HDMA3 & 0x1F) << 8) | (R_HDMA4 & 0xF0);
	int cnt = 16;
	while (cnt-- != 0)
	    mem_write(dst++, mem_read(src++));
	R_HDMA1 = (src >> 8) & 0xFF;
	R_HDMA2 = src & 0xF0;
	R_HDMA3 = (dst >> 8) & 0x1F;
	R_HDMA4 = dst & 0xF0;
	R_HDMA5--;
    }

    private final void hw_state_save(DataOutputStream dos) throws IOException {
	for (int i = 0; i < WRAM.length; i++)
	    dos.write(WRAM[i]);
	for (int i = 0; i < HIRAM.length; i++)
	    dos.write(HIRAM[i]);
	for (int i = 0; i < snd_regs.length; i++)
	    dos.write(snd_regs[i]);
	dos.write(R_P1);
	dos.write(R_SB);
	dos.write(R_SC);
	dos.write(R_DIV);
	dos.write(R_TIMA);
	dos.write(R_TMA);
	dos.write(R_TAC);
	dos.write(R_IF);
	dos.write(R_LCDC);
	dos.write(R_STAT);
	dos.write(R_SCY);
	dos.write(R_SCX);
	dos.write(R_LY);
	dos.write(R_LYC);
	dos.write(R_BGP);
	dos.write(R_OBP0);
	dos.write(R_OBP1);
	dos.write(R_WY);
	dos.write(R_WX);
	dos.write(R_KEY1);
	dos.write(R_VBK);
	dos.write(R_HDMA1);
	dos.write(R_HDMA2);
	dos.write(R_HDMA3);
	dos.write(R_HDMA4);
	dos.write(R_HDMA5);
	dos.write(R_RP);
	dos.write(R_BCPS);
	dos.write(R_OCPS);
	dos.write(R_SVBK);
	dos.write(R_IE);
    }

    private final void hw_state_load(DataInputStream dis) throws IOException {
	for (int i = 0; i < WRAM.length; i++)
	    dis.read(WRAM[i]);
	for (int i = 0; i < HIRAM.length; i++)
	    HIRAM[i] = dis.read() & 0xFF;
	for (int i = 0; i < snd_regs.length; i++)
	    snd_regs[i] = dis.read() & 0xFF;
	R_P1 = dis.read() & 0xFF;
	R_SB = dis.read() & 0xFF;
	R_SC = dis.read() & 0xFF;
	R_DIV = dis.read() & 0xFF;
	R_TIMA = dis.read() & 0xFF;
	R_TMA = dis.read() & 0xFF;
	R_TAC = dis.read() & 0xFF;
	R_IF = dis.read() & 0xFF;
	R_LCDC = dis.read() & 0xFF;
	R_STAT = dis.read() & 0xFF;
	R_SCY = dis.read() & 0xFF;
	R_SCX = dis.read() & 0xFF;
	R_LY = dis.read() & 0xFF;
	R_LYC = dis.read() & 0xFF;
	R_BGP = dis.read() & 0xFF;
	R_OBP0 = dis.read() & 0xFF;
	R_OBP1 = dis.read() & 0xFF;
	R_WY = dis.read() & 0xFF;
	R_WX = dis.read() & 0xFF;
	R_KEY1 = dis.read() & 0xFF;
	R_VBK = dis.read() & 0xFF;
	R_HDMA1 = dis.read() & 0xFF;
	R_HDMA2 = dis.read() & 0xFF;
	R_HDMA3 = dis.read() & 0xFF;
	R_HDMA4 = dis.read() & 0xFF;
	R_HDMA5 = dis.read() & 0xFF;
	R_RP = dis.read() & 0xFF;
	R_BCPS = dis.read() & 0xFF;
	R_OCPS = dis.read() & 0xFF;
	R_SVBK = dis.read() & 0xFF;
	R_IE = dis.read() & 0xFF;
    }


    // mem
    // nosave
    private byte[][] rmap;
    private byte[][] wmap;

    private final void mem_init() {
	rmap = new byte[16][];
	wmap = new byte[16][];
    }

    private final void mem_warmreset() {
	rmap[0] = ROM[0];
	rmap[1] = ROM[1];
	rmap[2] = ROM[2];
	rmap[3] = ROM[3];
	wmap[0xC] = rmap[0xC] = WRAM[0];

	mem_set_VRAM_bank();
	mem_set_WRAM_bank();

	mem_set_ROM_bank();
    }

    private final void mem_set_VRAM_bank() {
	rmap[8] = VRAM[(R_VBK & 1) << 1];
	rmap[9] = VRAM[((R_VBK & 1) << 1) + 1];
    }

    private final void mem_set_ROM_bank() {
	int b = (MR_ROMB << 2) % ROM.length;
	rmap[4] = ROM[b];
	rmap[5] = ROM[b + 1];
	rmap[6] = ROM[b + 2];
	rmap[7] = ROM[b + 3];

	b = (MR_RAMB << 1) % SRAM.length;
	if (MR_RAMG == 0x0A && (rtc_sel & 8) == 0) {
	    wmap[0xA] = rmap[0xA] = SRAM[b];
	    wmap[0xB] = rmap[0xB] = SRAM[b + 1];
	}
	else {
	    wmap[0xA] = rmap[0xA] = null;
	    wmap[0xB] = rmap[0xB] = null;
	}
    }

    private final void mem_set_WRAM_bank() {
	int n = R_SVBK & 7;
	wmap[0xD] = rmap[0xD] = WRAM[n != 0 ? n : 1];
    }

    private final int mem_read(int addr) {
	int page = addr >> 12, n;

	switch (page) {
	case 0:
	case 1:
	case 2:
	case 3:
	    return ROM[page][addr & 0x0FFF] & 0xFF;
	case 4:
	case 5:
	case 6:
	case 7:
	    return ROM[((MR_ROMB << 2) % ROM.length)+ (page & 3)][addr & 0x0FFF] & 0xFF;
	case 8:
	case 9:
	    return VRAM[((R_VBK & 1) << 1) + (page & 1)][addr & 0x0FFF] & 0xFF;
	case 0xA:
	case 0xB:
	    if (MR_RAMG == 0x0A)
		if ((rtc_sel & 8) != 0)
		    return rtc_read();
		else
		    return SRAM[((MR_RAMB << 1) % SRAM.length) + (page & 1)][addr & 0x0FFF] & 0xFF;
	    //return 0xFF;
	    return 0;
	case 0xC:
	    return WRAM[0][addr & 0x0FFF] & 0xFF;
	case 0xD:
	    n = R_SVBK & 7;
	    return WRAM[n != 0 ? n : 1][addr & 0x0FFF] & 0xFF;
	case 0xE:
	case 0xF:
	    if (addr < 0xFE00) // WRAM mirror
		return mem_read(addr - 0x2000);
	    if (addr < 0xFF00) // OAM
		return OAM[addr & 0xFF];
	    return mem_readhi(addr & 0xFF);

	default:
	    throw new Error("mem_read " + Integer.toHexString(addr));
	}
    }

    private final int mem_readhi(int addr) {
	if (addr >= 0x80 && addr != 0xFF)
	    return HIRAM[addr - 0x80];
	else
	    return hw_read(addr);
    }

    private final int mem_readw(int addr) {
	return mem_read(addr) | (mem_read(addr + 1) << 8);
    }

    private final void mem_write(int addr, int data) {
	int page = addr >> 12, n;
	switch (page) {
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	    mbc_write(addr, data);
	    return;
	case 8:
	    n = R_VBK & 1;
	    VRAM[n << 1][addr & 0x0FFF] = (byte)data;
	    patdirty[(n << 9) | ((addr >> 4) & 0xFF)] = true;
	    return;
	case 9:
	    n = R_VBK & 1;
	    VRAM[(n << 1) + 1][addr & 0x0FFF] = (byte)data;
	    if (addr < 0x9800)
		patdirty[(n << 9) | 0x100 | ((addr >> 4) & 0xFF)] = true;
	    return;
	case 0xA:
	case 0xB:
	    if (MR_RAMG == 0x0A)
		if ((rtc_sel & 8) != 0)
		    rtc_write(data);
		else
		    SRAM[((MR_RAMB << 1) % SRAM.length) + (page & 1)][addr & 0x0FFF] = (byte)data;
	    return;
	case 0xC:
	    WRAM[0][addr & 0x0FFF] = (byte)data;
	    return;
	case 0xD:
	    n = R_SVBK & 7;
	    WRAM[n != 0 ? n : 1][addr & 0x0FFF] = (byte)data;
	    return;
	case 0xE:
	case 0xF:
	    if (addr < 0xFE00) { // WRAM mirror
		mem_write(addr - 0x2000, data);
		return;
	    }
	    if (addr < 0xFF00) { // OAM
		OAM[addr & 0xFF] = data;
		return;
	    }
	    mem_writehi(addr & 0xFF, data);
	    return;

	default:
	    throw new Error("mem_write " + Integer.toHexString(addr) + " " + Integer.toHexString(data));
	}
    }

    private final void mem_writehi(int addr, int data) {
	if (addr < 0x80 || addr == 0xFF)
	    hw_write(addr, data);
	else
	    HIRAM[addr - 0x80] = data;
    }

    private final void mem_writew(int addr, int data) {
	mem_write(addr, data & 0xFF);
	mem_write(addr + 1, data >> 8);
    }


    // cpu
    private static final int F_Z = 0x80;
    private static final int F_N = 0x40;
    private static final int F_H = 0x20;
    private static final int F_C = 0x10;

    private static final int INT_VBLANK = 0x01;
    private static final int INT_LCDC = 0x02;
    private static final int INT_TIMER = 0x04;
    private static final int INT_SERIAL = 0x08;
    private static final int INT_PAD = 0x10;

    private static final int[] timer_clocks = { 1024, 16, 64, 256 };

    // save
    private int A;
    private int F;
    private int B;
    private int C;
    private int D;
    private int E;
    private int HL;
    private int SP;
    private int PC;

    private boolean cpu_halt;
    private boolean cpu_ime;
    private boolean int_disable;
    private int cpu_speed;

    //private int gdma_rest;
    private int rest_clock;
    private int sys_clock;
    private int div_clock;
    //private int total_clock;

    private final void cpu_init() {
    }

    private final void cpu_reset() {
	A = hw_type >= HW_GBC ? 0x11 : 0x01; // GB|Pbg0xFFɂȂ炵B
	F = 0xB0;
	B = 0x00;
	C = 0x13;
	D = 0x00;
	E = 0xD8;
	HL = 0x014D;
	SP = 0xFFFE;
	PC = 0x0100;

	cpu_halt = false;
	cpu_ime = false;
	int_disable = false;

	cpu_speed = 0;

	//gdma_rest = 0;
	rest_clock = 0;
	sys_clock = 0;
	div_clock = 0;
	//total_clock = 0;
    }

    private final void cpu_timers(int tmp_clocks) {
	rest_clock -= tmp_clocks;
	div_clock += tmp_clocks;
	//total_clock += tmp_clocks;
	
	if ((R_TAC & 4) != 0) {
	    sys_clock += tmp_clocks;
	    if (sys_clock >= timer_clocks[R_TAC & 3]) {
		sys_clock &= timer_clocks[R_TAC & 3] - 1;
		R_TIMA = (R_TIMA + 1) & 0xFF;
		if (R_TIMA == 0) {
		    R_IF |= INT_TIMER;
		    R_TIMA = R_TMA;
		}
	    }
	}
	
	if (div_clock >= 0x100) {
	    R_DIV = (R_DIV - (div_clock >> 8)) & 0xFF;
	    div_clock &= 0xFF;
	}
	
	// serial
    }

    private final void cpu_irq_process() {
	if (int_disable) {
	    int_disable = false;
	    return;
	}

	if ((R_IF & R_IE) != 0 && (cpu_ime || cpu_halt)) {
	    //if (cpu_halt)
	    //PC++;
	    
	    SP -= 2; mem_writew(SP, PC);
	    if ((R_IF &  R_IE & INT_VBLANK) != 0) {
		PC = 0x40;
		R_IF &= 0xFE;
	    }
	    else if ((R_IF &  R_IE & INT_LCDC) != 0) {
		PC = 0x48;
		R_IF &= 0xFD;
	    }
	    else if ((R_IF &  R_IE & INT_TIMER) != 0) {
		PC = 0x50;
		R_IF &= 0xFB;
	    }
	    else if ((R_IF &  R_IE & INT_SERIAL) != 0) {
		PC = 0x58;
		R_IF &= 0xF7;
	    }
	    else if ((R_IF &  R_IE & INT_PAD) != 0) {
		PC = 0x60;
		R_IF &= 0xEF;
	    }
	    
	    cpu_halt = false;
	    cpu_ime = false;
	}
    }

    private final int cpu_next_event(int rest_clock) {
	if ((R_TAC & 4) != 0) {
	    int tc = (0x100 - R_TIMA) * timer_clocks[R_TAC & 3] - sys_clock;
	    if (rest_clock >= tc)
		rest_clock = tc;
	}

	/*
	int dc = 0x100 - div_clock;
	if (rest_clock >= dc)
	    rest_clock = dc;
	*/

	return rest_clock;
    }

    private final void cpu_exec(int cycles) { // GB4.194MHzNbNB
	if (cpu_speed != 0)
	    cycles *= 2;

	rest_clock += cycles;

	//if (gdma_rest != 0) {
	//if (rest_clock <= gdma_rest) {
	//gdma_rest -= rest_clock;
	//sys_clock += rest_clock;
	//div_clock += rest_clock;
	////total_clock += rest_clock;
	//rest_clock = 0;
	//}
	//else {
	//rest_clock -= gdma_rest;
	//sys_clock += gdma_rest;
	//div_clock += gdma_rest;
	////total_clock += gdma_rest;
	//gdma_rest = 0;
	//}
	//}

	while (rest_clock > 0) {
	    cpu_irq_process();
	    int c = cpu_exec1(cpu_next_event(rest_clock));
	    cpu_timers(c);
	}
    }

    private final int cpu_exec1(int rest_clock) {
	if (cpu_halt)
	    return rest_clock;

	int rest_clock_ = rest_clock;

	int A = this.A;
	int B = this.B;
	int C = this.C;
	int D = this.D;
	int HL = this.HL;
	int SP = this.SP;

	int acc = (~F & F_Z) | ((F & F_C) << 4);
	int aux = F >> 1;
	int nega = (F >> 6) & 1;

	L0: for (;;) {

	    int PChi = PC & 0xF000;
	    int PClo = PC & 0x0FFF;
	    byte[] curpage = rmap[PChi >> 12];
	    int SPhi = SP & 0xF000;
	    int SPlo = SP & 0x0FFF;
	    byte[] stkpage = rmap[SPhi >> 12];

	    int lastPClo = 0;

	    try {
		L1: while (rest_clock > 0) {

		    lastPClo = PClo;
		    int op = curpage[PClo++] & 0xFF, tmp_clocks = 0;

		    switch (op) {
			// include begin
			
		    case 0x00: // NOP
				tmp_clocks = 4;
				break;
		    case 0x40: // LD B,B
				tmp_clocks = 4;
				break;
		    case 0x41: // LD B,C
				B = C;
				tmp_clocks = 4;
				break;
		    case 0x42: // LD B,D
				B = D;
				tmp_clocks = 4;
				break;
		    case 0x43: // LD B,E
				B = E;
				tmp_clocks = 4;
				break;
		    case 0x44: // LD B,H
				B = ((HL >> 8) & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x45: // LD B,L
				B = (HL & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x46: // LD B,(HL)
				B = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
				tmp_clocks = 8;
				break;
		    case 0x47: // LD B,A
				B = A;
				tmp_clocks = 4;
				break;
		    case 0x48: // LD C,B
				C = B;
				tmp_clocks = 4;
				break;
		    case 0x49: // LD C,C
				tmp_clocks = 4;
				break;
		    case 0x4A: // LD C,D
				C = D;
				tmp_clocks = 4;
				break;
		    case 0x4B: // LD C,E
				C = E;
				tmp_clocks = 4;
				break;
		    case 0x4C: // LD C,H
				C = ((HL >> 8) & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x4D: // LD C,L
				C = (HL & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x4E: // LD C,(HL)
				C = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
				tmp_clocks = 8;
				break;
		    case 0x4F: // LD C,A
				C = A;
				tmp_clocks = 4;
				break;
		    case 0x50: // LD D,B
				D = B;
				tmp_clocks = 4;
				break;
		    case 0x51: // LD D,C
				D = C;
				tmp_clocks = 4;
				break;
		    case 0x52: // LD D,D
				tmp_clocks = 4;
				break;
		    case 0x53: // LD D,E
				D = E;
				tmp_clocks = 4;
				break;
		    case 0x54: // LD D,H
				D = ((HL >> 8) & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x55: // LD D,L
				D = (HL & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x56: // LD D,(HL)
				D = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
				tmp_clocks = 8;
				break;
		    case 0x57: // LD D,A
				D = A;
				tmp_clocks = 4;
				break;
		    case 0x58: // LD E,B
				E = B;
				tmp_clocks = 4;
				break;
		    case 0x59: // LD E,C
				E = C;
				tmp_clocks = 4;
				break;
		    case 0x5A: // LD E,D
				E = D;
				tmp_clocks = 4;
				break;
		    case 0x5B: // LD E,E
				tmp_clocks = 4;
				break;
		    case 0x5C: // LD E,H
				E = ((HL >> 8) & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x5D: // LD E,L
				E = (HL & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x5E: // LD E,(HL)
				E = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
				tmp_clocks = 8;
				break;
		    case 0x5F: // LD E,A
				E = A;
				tmp_clocks = 4;
				break;
		    case 0x60: // LD H,B
				HL = ((B) << 8) | (HL & 0x00FF);
				tmp_clocks = 4;
				break;
		    case 0x61: // LD H,C
				HL = ((C) << 8) | (HL & 0x00FF);
				tmp_clocks = 4;
				break;
		    case 0x62: // LD H,D
				HL = ((D) << 8) | (HL & 0x00FF);
				tmp_clocks = 4;
				break;
		    case 0x63: // LD H,E
				HL = ((E) << 8) | (HL & 0x00FF);
				tmp_clocks = 4;
				break;
		    case 0x64: // LD H,H
				tmp_clocks = 4;
				break;
		    case 0x65: // LD H,L
				HL = (((HL & 0xFF)) << 8) | (HL & 0x00FF);
				tmp_clocks = 4;
				break;
		    case 0x66: // LD H,(HL)
				HL = (((rmap[HL >> 12][HL & 0x0FFF] & 0xFF)) << 8) | (HL & 0x00FF);
				tmp_clocks = 8;
				break;
		    case 0x67: // LD H,A
				HL = ((A) << 8) | (HL & 0x00FF);
				tmp_clocks = 4;
				break;
		    case 0x68: // LD L,B
				HL = (HL & 0xFF00) | (B);
				tmp_clocks = 4;
				break;
		    case 0x69: // LD L,C
				HL = (HL & 0xFF00) | (C);
				tmp_clocks = 4;
				break;
		    case 0x6A: // LD L,D
				HL = (HL & 0xFF00) | (D);
				tmp_clocks = 4;
				break;
		    case 0x6B: // LD L,E
				HL = (HL & 0xFF00) | (E);
				tmp_clocks = 4;
				break;
		    case 0x6C: // LD L,H
				HL = (HL & 0xFF00) | (((HL >> 8) & 0xFF));
				tmp_clocks = 4;
				break;
		    case 0x6D: // LD L,L
				tmp_clocks = 4;
				break;
		    case 0x6E: // LD L,(HL)
				HL = (HL & 0xFF00) | ((rmap[HL >> 12][HL & 0x0FFF] & 0xFF));
				tmp_clocks = 8;
				break;
		    case 0x6F: // LD L,A
				HL = (HL & 0xFF00) | (A);
				tmp_clocks = 4;
				break;
		    case 0x70: // LD (HL),B
				wmap[HL >> 12][HL & 0x0FFF] = (byte)B; 
				tmp_clocks = 8;
				break;
		    case 0x71: // LD (HL),C
				wmap[HL >> 12][HL & 0x0FFF] = (byte)C; 
				tmp_clocks = 8;
				break;
		    case 0x72: // LD (HL),D
				wmap[HL >> 12][HL & 0x0FFF] = (byte)D; 
				tmp_clocks = 8;
				break;
		    case 0x73: // LD (HL),E
				wmap[HL >> 12][HL & 0x0FFF] = (byte)E; 
				tmp_clocks = 8;
				break;
		    case 0x74: // LD (HL),H
				wmap[HL >> 12][HL & 0x0FFF] = (byte)((HL >> 8) & 0xFF); 
				tmp_clocks = 8;
				break;
		    case 0x75: // LD (HL),L
				wmap[HL >> 12][HL & 0x0FFF] = (byte)(HL & 0xFF); 
				tmp_clocks = 8;
				break;
		    case 0x77: // LD (HL),A
				wmap[HL >> 12][HL & 0x0FFF] = (byte)A; 
				tmp_clocks = 8;
				break;
		    case 0x78: // LD A,B
				A = B;
				tmp_clocks = 4;
				break;
		    case 0x79: // LD A,C
				A = C;
				tmp_clocks = 4;
				break;
		    case 0x7A: // LD A,D
				A = D;
				tmp_clocks = 4;
				break;
		    case 0x7B: // LD A,E
				A = E;
				tmp_clocks = 4;
				break;
		    case 0x7C: // LD A,H
				A = ((HL >> 8) & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x7D: // LD A,L
				A = (HL & 0xFF);
				tmp_clocks = 4;
				break;
		    case 0x7E: // LD A,(HL)
				A = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
				tmp_clocks = 8;
				break;
		    case 0x7F: // LD A,A
				tmp_clocks = 4;
				break;
		    case 0x01: // LD BC,imm
				{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; B = (t >> 8) & 0xFF; C = t & 0xFF; }
				tmp_clocks = 12;
				break;
		    case 0x11: // LD DE,imm
				{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; D = (t >> 8) & 0xFF; E = t & 0xFF; }
				tmp_clocks = 12;
				break;
		    case 0x21: // LD HL,imm
				{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; HL = t;}
				tmp_clocks = 12;
				break;
		    case 0x31: // LD SP,imm
				{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; SPhi = t & 0xF000; SPlo = t & 0x0FFF; stkpage = wmap[SPhi >> 12]; }
				tmp_clocks = 12;
				break;
		    case 0x02: // LD (BC),A
				{ int t = ((B << 8) | C);
				wmap[t >> 12][t & 0x0FFF] = (byte)A; }
				tmp_clocks = 8;
				break;
		    case 0x12: // LD (DE),A
				{ int t = ((D << 8) | E);
				wmap[t >> 12][t & 0x0FFF] = (byte)A; }
				tmp_clocks = 8;
				break;
		    case 0x22: // LDI (HL),A
				wmap[HL >> 12][HL & 0x0FFF] = (byte)A; HL++;
				tmp_clocks = 8;
				break;
		    case 0x32: // LDD (HL),A
				wmap[HL >> 12][HL & 0x0FFF] = (byte)A; HL--;
				tmp_clocks = 8;
				break;
		    case 0x0A: // LD A,(BC)
			{ int t = ((B << 8) | C);
			A = (rmap[t >> 12][t & 0x0FFF] & 0xFF); }
			tmp_clocks = 8;
			break;
		    case 0x1A: // LD A,(DE)
			{ int t = ((D << 8) | E);
			A = (rmap[t >> 12][t & 0x0FFF] & 0xFF); }
			tmp_clocks = 8;
			break;
		    case 0x2A: // LDI A,(HL)
			A = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); HL++;
			tmp_clocks = 8;
			break;
		    case 0x3A: // LDD A,(HL))
			A = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); HL--;
			tmp_clocks = 8;
			break;
		    case 0x06: // LD B,imm
			{ int t = curpage[PClo++] & 0xFF; B = t;}
			tmp_clocks = 8;
			break;
		    case 0x0E: // LD C,imm
			{ int t = curpage[PClo++] & 0xFF; C = t;}
			tmp_clocks = 8;
			break;
		    case 0x16: // LD D,imm
			{ int t = curpage[PClo++] & 0xFF; D = t;}
			tmp_clocks = 8;
			break;
		    case 0x1E: // LD E,imm
			{ int t = curpage[PClo++] & 0xFF; E = t;}
			tmp_clocks = 8;
			break;
		    case 0x26: // LD H,imm
			{ int t = curpage[PClo++] & 0xFF; HL = ((t) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x2E: // LD L,imm
			{ int t = curpage[PClo++] & 0xFF; HL = (HL & 0xFF00) | (t);}
			tmp_clocks = 8;
			break;
		    case 0x36: // LD (HL),imm
			{ int t = curpage[PClo++] & 0xFF; wmap[HL >> 12][HL & 0x0FFF] = (byte)t; }
			tmp_clocks = 12;
			break;
		    case 0x3E: // LD A,imm
			{ int t = curpage[PClo++] & 0xFF; A = t;}
			tmp_clocks = 8;
			break;
		    case 0x08: // LD (imm),SP
			{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; mem_writew(t, (SPhi + SPlo)); }
			tmp_clocks = 20;
			break;
		    case 0xEA: // LD (imm),A
			{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; wmap[t >> 12][t & 0x0FFF] = (byte)A; }
			tmp_clocks = 16;
			break;
		    case 0xE0: // LDH (imm),A
			{ int t = curpage[PClo++] & 0xFF; mem_writehi(t, A); }
			tmp_clocks = 12;
			break;

		    case 0xE2: // LDH (C),A
			mem_writehi(C, A);
			tmp_clocks = 8;
			break;
		    case 0xF0: // LDH A,(imm)
			{ int t = curpage[PClo++] & 0xFF; A = mem_readhi(t); }
			tmp_clocks = 12;
			break;
		    case 0xF2: // LDH A,(C)
			A = mem_readhi(C);
			tmp_clocks = 8;
			break;
		    case 0xF8: // LD HL,SP+imm
			{ int n = curpage[PClo++]; 
			int _SP = (SPhi + SPlo); 
			int t = _SP + n;
			acc = (t >> 8) | 1;
			aux = ((t ^ _SP ^ n) >> 8) ^ acc; nega = 0;
			HL = t & 0xFFFF;}
			tmp_clocks = 12;
			break;
		    case 0xF9: // LD SP,HL
			SPhi = HL & 0xF000; SPlo = HL & 0x0FFF; stkpage = wmap[SPhi >> 12]; 
			tmp_clocks = 8;
			break;
		    case 0xFA: // LD A,(imm)
			{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; A = (rmap[t >> 12][t & 0x0FFF] & 0xFF); }
			tmp_clocks = 16;
			break;
		    case 0xC6: // ADD A,imm
			{ int n = curpage[PClo++] & 0xFF; 
			acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x80: // ADD A,B
			{
			    acc = A + B; aux = A ^ B; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x81: // ADD A,C
			{
			    acc = A + C; aux = A ^ C; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
			
		    case 0x82: // ADD A,D
			{
			    acc = A + D; aux = A ^ D; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x83: // ADD A,E
			{
			    acc = A + E; aux = A ^ E; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x84: // ADD A,H
			{ int n = ((HL >> 8) & 0xFF);
			acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x85: // ADD A,L
			{ int n = (HL & 0xFF);
			acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x86: // ADD A,(HL)
			{ int n = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
			acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x87: // ADD A,A
			{
			    acc = A + A; aux = A ^ A; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0xCE: // ADC A,imm
			{ int n = curpage[PClo++] & 0xFF; 
			acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x88: // ADC A,B
			{
			    acc = A + B + ((acc >> 8) & 1); aux = A ^ B; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x89: // ADC A,C
			{
			    acc = A + C + ((acc >> 8) & 1); aux = A ^ C; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x8A: // ADC A,D
			{
			    acc = A + D + ((acc >> 8) & 1); aux = A ^ D; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x8B: // ADC A,E
			{
			    acc = A + E + ((acc >> 8) & 1); aux = A ^ E; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x8C: // ADC A,H
			{ int n = ((HL >> 8) & 0xFF);
			acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x8D: // ADC A,L
			{ int n = (HL & 0xFF);
			acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x8E: // ADC A,(HL)
			{ int n = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
			acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x8F: // ADC A,A
			{
			    acc = A + A + ((acc >> 8) & 1); aux = A ^ A; nega = 0; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0xD6: // SUB imm
			{ int n = curpage[PClo++] & 0xFF; 
			acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x90: // SUB B
			{
			    acc = A - B; aux = A ^ B; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x91: // SUB C
			{
			    acc = A - C; aux = A ^ C; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x92: // SUB D
			{
			    acc = A - D; aux = A ^ D; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x93: // SUB E
			{
			    acc = A - E; aux = A ^ E; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x94: // SUB H
			{ int n = ((HL >> 8) & 0xFF);
			acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x95: // SUB L
			{ int n = (HL & 0xFF);
			acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x96: // SUB (HL)
			{ int n = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
			acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x97: // SUB A
			{
			    acc = A - A; aux = A ^ A; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0xDE: // SBC A,imm
			{ int n = curpage[PClo++] & 0xFF; 
			acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x98: // SBC A,B
			{
			    acc = A - B - ((acc >> 8) & 1); aux = A ^ B; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x99: // SBC A,C
			{
			    acc = A - C - ((acc >> 8) & 1); aux = A ^ C; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x9A: // SBC A,D
			{
			    acc = A - D - ((acc >> 8) & 1); aux = A ^ D; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x9B: // SBC A,E
			{
			    acc = A - E - ((acc >> 8) & 1); aux = A ^ E; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x9C: // SBC A,H
			{ int n = ((HL >> 8) & 0xFF);
			acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x9D: // SBC A,L
			{ int n = (HL & 0xFF);
			acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0x9E: // SBC A,(HL)
			{ int n = (rmap[HL >> 12][HL & 0x0FFF] & 0xFF);
			acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 8;
			break;
		    case 0x9F: // SBC A,A
			{
			    acc = A - A - ((acc >> 8) & 1); aux = A ^ A; nega = 1; A = acc & 0xFF;
			}
			tmp_clocks = 4;
			break;
		    case 0xE6: // AND imm
			{ int n = curpage[PClo++] & 0xFF; 
			A = acc = A & n; aux = ~acc; nega = 0;
			}
			tmp_clocks = 8;
			break;
		    case 0xA0: // AND B
			{
			    A = acc = A & B; aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA1: // AND C
			{
			    A = acc = A & C; aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA2: // AND D
			{
			    A = acc = A & D; aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA3: // AND E
			{
			    A = acc = A & E; aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA4: // AND H
			{
			    A = acc = A & ((HL >> 8) & 0xFF); aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA5: // AND L
			{
			    A = acc = A & (HL & 0xFF); aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA6: // AND (HL)
			{
			    A = acc = A & (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); aux = ~acc; nega = 0;
			}
			tmp_clocks = 8;
			break;
		    case 0xA7: // AND A
			{
			    A = acc = A & A; aux = ~acc; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xEE: // XOR imm
			{ int n = curpage[PClo++] & 0xFF; 
			A = aux = acc = A ^ n; nega = 0;
			}
			tmp_clocks = 8;
			break;
		    case 0xA8: // XOR B
			{
			    A = aux = acc = A ^ B; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xA9: // XOR C
			{
			    A = aux = acc = A ^ C; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xAA: // XOR D
			{
			    A = aux = acc = A ^ D; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xAB: // XOR E
			{
			    A = aux = acc = A ^ E; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xAC: // XOR H
			{
			    A = aux = acc = A ^ ((HL >> 8) & 0xFF); nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xAD: // XOR L
			{
			    A = aux = acc = A ^ (HL & 0xFF); nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xAE: // XOR (HL)
			{
			    A = aux = acc = A ^ (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); nega = 0;
			}
			tmp_clocks = 8;
			break;
		    case 0xAF: // XOR A
			{
			    A = aux = acc = A ^ A; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xF6: // OR imm
			{ int n = curpage[PClo++] & 0xFF; 
			A = aux = acc = A | n; nega = 0;
			}
			tmp_clocks = 8;
			break;
		    case 0xB0: // OR B
			{
			    A = aux = acc = A | B; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xB1: // OR C
			{
			    A = aux = acc = A | C; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xB2: // OR D
			{
			    A = aux = acc = A | D; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xB3: // OR E
			{
			    A = aux = acc = A | E; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xB4: // OR H
			{
			    A = aux = acc = A | ((HL >> 8) & 0xFF); nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xB5: // OR L
			{
			    A = aux = acc = A | (HL & 0xFF); nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xB6: // OR (HL)
			{
			    A = aux = acc = A | (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); nega = 0;
			}
			tmp_clocks = 8;
			break;
		    case 0xB7: // OR A
			{
			    A = aux = acc = A | A; nega = 0;
			}
			tmp_clocks = 4;
			break;
		    case 0xFE: // CP imm
			{ int n = curpage[PClo++] & 0xFF; 
			acc = A - n; aux = A ^ n; nega = 1;
			}
			tmp_clocks = 8;
			break;
		    case 0xB8: // CP B
			{
			    acc = A - B; aux = A ^ B; nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0xB9: // CP C
			{
			    acc = A - C; aux = A ^ C; nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0xBA: // CP D
			{
			    acc = A - D; aux = A ^ D; nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0xBB: // CP E
			{
			    acc = A - E; aux = A ^ E; nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0xBC: // CP H
			{
			    acc = A - ((HL >> 8) & 0xFF); aux = A ^ ((HL >> 8) & 0xFF); nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0xBD: // CP L
			{
			    acc = A - (HL & 0xFF); aux = A ^ (HL & 0xFF); nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0xBE: // CP (HL)
			{
			    acc = A - (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); aux = A ^ (rmap[HL >> 12][HL & 0x0FFF] & 0xFF); nega = 1;
			}
			tmp_clocks = 8;
			break;
		    case 0xBF: // CP A
			{
			    acc = A - A; aux = A ^ A; nega = 1;
			}
			tmp_clocks = 4;
			break;
		    case 0x09: // ADD HL,BC
			{ int n = ((B << 8) | C);
			int t = HL + n;
			acc = ((t >> 8) & 0x100) | (acc & 0xFF);
			aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
			HL = t & 0xFFFF;}
			tmp_clocks = 8;
			break;
		    case 0x19: // ADD HL,DE
			{ int n = ((D << 8) | E);
			int t = HL + n;
			acc = ((t >> 8) & 0x100) | (acc & 0xFF);
			aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
			HL = t & 0xFFFF;}
			tmp_clocks = 8;
			break;
		    case 0x29: // ADD HL,HL
			{ int n = HL;
			int t = HL + n;
			acc = ((t >> 8) & 0x100) | (acc & 0xFF);
			aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
			HL = t & 0xFFFF;}
			tmp_clocks = 8;
			break;
		    case 0x39: // ADD HL,SP
			{ int n = (SPhi + SPlo);
			int t = HL + n;
			acc = ((t >> 8) & 0x100) | (acc & 0xFF);
			aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
			HL = t & 0xFFFF;}
			tmp_clocks = 8;
			break;
		    case 0x04: // INC B
			aux = B; nega = 0; B = (B + 1) & 0xFF; acc = B | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x0C: // INC C
			aux = C; nega = 0; C = (C + 1) & 0xFF; acc = C | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x14: // INC D
			aux = D; nega = 0; D = (D + 1) & 0xFF; acc = D | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x1C: // INC E
			aux = E; nega = 0; E = (E + 1) & 0xFF; acc = E | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x24: // INC H
			{ int n = ((HL >> 8) & 0xFF);
			aux = n; nega = 0; n = (n + 1) & 0xFF; acc = n | (acc & 0x100);
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 4;
			break;
		    case 0x2C: // INC L
			{ int n = (HL & 0xFF);
			aux = n; nega = 0; n = (n + 1) & 0xFF; acc = n | (acc & 0x100);
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 4;
			break;
		    case 0x34: // INC (HL)
			{ byte[] p = wmap[HL >> 12]; int t = HL & 0x0FFF; int n = p[t] & 0xFF; 
			aux = n; nega = 0; n = (n + 1) & 0xFF; acc = n | (acc & 0x100);
			p[t] = (byte)n; }
			tmp_clocks = 12;
			break;
		    case 0x3C: // INC A
			aux = A; nega = 0; A = (A + 1) & 0xFF; acc = A | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x05: // DEC B
			aux = B; nega = 1; B = (B - 1) & 0xFF; acc = B | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x0D: // DEC C
			aux = C; nega = 1; C = (C - 1) & 0xFF; acc = C | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x15: // DEC D
			aux = D; nega = 1; D = (D - 1) & 0xFF; acc = D | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x1D: // DEC E
			aux = E; nega = 1; E = (E - 1) & 0xFF; acc = E | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x25: // DEC H
			{ int n = ((HL >> 8) & 0xFF);
			aux = n; nega = 1; n = (n - 1) & 0xFF; acc = n | (acc & 0x100);
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 4;
			break;
		    case 0x2D: // DEC L
			{ int n = (HL & 0xFF);
			aux = n; nega = 1; n = (n - 1) & 0xFF; acc = n | (acc & 0x100);
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 4;
			break;
		    case 0x35: // DEC (HL)
			{ byte[] p = wmap[HL >> 12]; int t = HL & 0x0FFF; int n = p[t] & 0xFF; 
			aux = n; nega = 1; n = (n - 1) & 0xFF; acc = n | (acc & 0x100);
			p[t] = (byte)n; }
			tmp_clocks = 12;
			break;
		    case 0x3D: // DEC A
			aux = A; nega = 1; A = (A - 1) & 0xFF; acc = A | (acc & 0x100);
			tmp_clocks = 4;
			break;
		    case 0x03: // INC BC
			{ int s = ((B << 8) | C) + 1; B = (s >> 8) & 0xFF; C = s & 0xFF; } 
			tmp_clocks = 8;
			break;
		    case 0x13: // INC DE
			{ int s = ((D << 8) | E) + 1; D = (s >> 8) & 0xFF; E = s & 0xFF; } 
			tmp_clocks = 8;
			break;
		    case 0x23: // INC HL
			HL = (HL + 1) & 0xFFFF;
			tmp_clocks = 8;
			break;
		    case 0x33: // INC SP
			SPhi = (SPhi + SPlo) + 1 & 0xF000; SPlo = (SPhi + SPlo) + 1 & 0x0FFF; stkpage = wmap[SPhi >> 12]; 
			tmp_clocks = 8;
			break;
		    case 0x0B: // DEC BC
			{ int s = ((B << 8) | C) - 1; B = (s >> 8) & 0xFF; C = s & 0xFF; } 
			tmp_clocks = 8;
			break;
		    case 0x1B: // DEC DE
			{ int s = ((D << 8) | E) - 1; D = (s >> 8) & 0xFF; E = s & 0xFF; } 
			tmp_clocks = 8;
			break;
		    case 0x2B: // DEC HL
			HL = (HL - 1) & 0xFFFF;
			tmp_clocks = 8;
			break;
		    case 0x3B: // DEC SP
			SPhi = (SPhi + SPlo) - 1 & 0xF000; SPlo = (SPhi + SPlo) - 1 & 0x0FFF; stkpage = wmap[SPhi >> 12]; 
			tmp_clocks = 8;
			break;
		    case 0x07: // RLCA
			aux = acc = (A << 1) | (A >> 7); A = acc & 0xFF; nega = 0;
			acc |= 1;
			tmp_clocks = 4;
			break;
		    case 0x0F: // RRCA
			aux = acc = ((A & 1) << 7) | (A >> 1) | (A << 8); A = acc & 0xFF; nega = 0;
			acc |= 1;
			tmp_clocks = 4;
			break;
		    case 0x17: // RLA
			aux = acc = (A << 1) | ((acc >> 8) & 1); A = acc & 0xFF; nega = 0;
			acc |= 1;
			tmp_clocks = 4;
			break;
		    case 0x1F: // RRA
			aux = acc = ((acc >> 1) & 0x80) | (A >> 1) | (A << 8); A = acc & 0xFF; nega = 0;
			acc |= 1;
			tmp_clocks = 4;
			break;
		    case 0x27: // DAA
			{ int n = daa(((packF(acc, aux, nega) & 0x70) << 4) | A);
			A = (A + n) & 0xFF;
			acc = (daa_carry(n >> 2) << 4) | A; aux = ~acc; }
			tmp_clocks = 4;
			break;
		    case 0x2F: // CPL
			A = ~A & 0xFF;
			aux = ~acc; nega = 1;
			tmp_clocks = 4;
			break;
		    case 0x18: // JR
			{ int t = curpage[PClo++]; PClo += t; }
			tmp_clocks = 12;
			break;
		    case 0x20: // JR NZ
			if ((acc & 0xFF) != 0) {
			    int t = curpage[PClo++]; 
			    PClo += t; 
			    tmp_clocks = 12;
			    break;
			} else {
			    PClo += 1; 
			    tmp_clocks = 8;
			    break;
			}
		    case 0x28: // JR Z
			if ((acc & 0xFF) == 0) {
			    int t = curpage[PClo++]; 
			    PClo += t; 
			    tmp_clocks = 12;
			    break;
			} else {
			    PClo += 1; 
			    tmp_clocks = 8;
			    break;
			}
		    case 0x30: // JR NC
			if ((acc & 0x100) == 0) {
			    int t = curpage[PClo++]; 
			    PClo += t; 
			    tmp_clocks = 12;
			    break;
			} else {
			    PClo += 1; 
			    tmp_clocks = 8;
			    break;
			}
		    case 0x38: // JR C
			if ((acc & 0x100) != 0) {
			    int t = curpage[PClo++]; 
			    PClo += t; 
			    tmp_clocks = 12;
			    break;
			} else {
			    PClo += 1; 
			    tmp_clocks = 8;
			    break;
			}
		    case 0xC3: // JP
			{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; }
			tmp_clocks = 16;
			break;
		    case 0xC2: // JP NZ
			if ((acc & 0xFF) != 0) {
			    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 16;
			    break;
			} else {
			    PClo += 2; 
			    tmp_clocks = 12;
			    break;
			}
		    case 0xCA: // JP Z
			if ((acc & 0xFF) == 0) {
			    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 16;
			    break;
			} else {
			    PClo += 2; 
			    tmp_clocks = 12;
			    break;
			}
		    case 0xD2: // JP NC
			if ((acc & 0x100) == 0) {
			    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 16;
			    break;
			} else {
			    PClo += 2; 
			    tmp_clocks = 12;
			    break;
			}
		    case 0xDA: // JP C
			if ((acc & 0x100) != 0) {
			    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 16;
			    break;
			} else {
			    PClo += 2; 
			    tmp_clocks = 12;
			    break;
			}
		    case 0xE9: // JP (HL)
			PChi = HL & 0xF000; PClo = HL & 0x0FFF; curpage = rmap[PChi >> 12]; 
			tmp_clocks = 4;
			break;
		    case 0xC9: // RET
			{ int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; }
			tmp_clocks = 16;
			break;
		    case 0xC0: // RET NZ
			if ((acc & 0xFF) != 0) {
			    int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 20;
			    break;
			} else {
			    tmp_clocks = 8;
			    break;
			}
		    case 0xC8: // RET Z
			if ((acc & 0xFF) == 0) {
			    int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 20;
			    break;
			} else {
			    tmp_clocks = 8;
			    break;
			}
		    case 0xD0: // RET NC
			if ((acc & 0x100) == 0) {
			    int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; 
			    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
			    tmp_clocks = 20;
			    break;
			} else {
			    tmp_clocks = 8;
			    break;
			}
		    case 0xD8: // RET C
				if ((acc & 0x100) != 0) {
				    int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; 
				    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
				    tmp_clocks = 20;
				    break;
				} else {
				    tmp_clocks = 8;
				    break;
				}
		    case 0xD9: // RETI
				cpu_ime = int_disable = true;
				{ int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; }
				tmp_clocks = 16;
				break;
		    case 0xCD: // CALL
				{ int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; { int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; }
				tmp_clocks = 24;
				break;
		    case 0xC4: // CALL NZ
				if ((acc & 0xFF) != 0) {
				    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
				    { int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
				    tmp_clocks = 24;
				    break;
				} else {
				    PClo += 2; 
				    tmp_clocks = 12;
				    break;
				}
		    case 0xCC: // CALL Z
				if ((acc & 0xFF) == 0) {
				    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
				    { int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
				    tmp_clocks = 24;
				    break;
				} else {
				    PClo += 2; 
				    tmp_clocks = 12;
				    break;
				}
		    case 0xD4: // CALL NC
				if ((acc & 0x100) == 0) {
				    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
				    { int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
				    tmp_clocks = 24;
				    break;
				} else {
				    PClo += 2; 
				    tmp_clocks = 12;
				    break;
				}
		    case 0xDC: // CALL C
				if ((acc & 0x100) != 0) {
				    int t = (curpage[PClo] & 0xFF) | ((curpage[PClo + 1] & 0xFF) << 8); PClo+= 2; 
				    { int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				    PChi = t & 0xF000; PClo = t & 0x0FFF; curpage = rmap[PChi >> 12]; 
				    tmp_clocks = 24;
				    break;
				} else {
				    PClo += 2; 
				    tmp_clocks = 12;
				    break;
				}
		    case 0xC7: // RST 0
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 0 & 0xF000; PClo = 0 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xCF: // RST 8
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 8 & 0xF000; PClo = 8 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xD7: // RST 10
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 16 & 0xF000; PClo = 16 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xDF: // RST 18
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 24 & 0xF000; PClo = 24 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xE7: // RST 20
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 32 & 0xF000; PClo = 32 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xEF: // RST 28
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 40 & 0xF000; PClo = 40 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xF7: // RST 30
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 48 & 0xF000; PClo = 48 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xFF: // RST 38
				{ int tpc = PChi + PClo; stkpage[SPlo - 2] = (byte)tpc; stkpage[SPlo - 1] = (byte)(tpc >> 8); SPlo -= 2; } 
				PChi = 56 & 0xF000; PClo = 56 & 0x0FFF; curpage = rmap[PChi >> 12]; 
				tmp_clocks = 16;
				break;
		    case 0xC1: // POP BC
				{ int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; B = (t >> 8) & 0xFF; C = t & 0xFF; }
				tmp_clocks = 12;
				break;
		    case 0xD1: // POP DE
				{ int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; D = (t >> 8) & 0xFF; E = t & 0xFF; }
				tmp_clocks = 12;
				break;
		    case 0xE1: // POP HL
				HL = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; 
				tmp_clocks = 12;
				break;
		    case 0xF1: // POP AF
				{ int t = (stkpage[SPlo] & 0xFF) | ((stkpage[SPlo + 1] & 0xFF) << 8); SPlo += 2; 
				A = (t >> 8) & 0xFF;
				acc = (~t & 0x80) | ((t & 0x10) << 4);
				aux = t >> 1;  nega = (t >> 6) & 1; }
				tmp_clocks = 12;
				break;
		    case 0xC5: // PUSH BC
				stkpage[SPlo - 2] = (byte)C; stkpage[SPlo - 1] = (byte)B; SPlo -= 2; 
				tmp_clocks = 16;
				break;
		    case 0xD5: // PUSH DE
				stkpage[SPlo - 2] = (byte)E; stkpage[SPlo - 1] = (byte)D; SPlo -= 2; 
				tmp_clocks = 16;
				break;
		    case 0xE5: // PUSH HL
				stkpage[SPlo - 2] = (byte)HL; stkpage[SPlo - 1] = (byte)(HL >> 8); SPlo -= 2; 
				tmp_clocks = 16;
				break;
		    case 0xF5: // PUSH AF
				stkpage[SPlo - 2] = (byte)(packF(acc, aux, nega) | 0x0E); stkpage[SPlo - 1] = (byte)A; SPlo -= 2; 
				tmp_clocks = 16;
				break;
		    case 0xE8: // ADD SP,imm
				{ int n = curpage[PClo++]; 
				int _SP = (SPhi + SPlo); 
				int t = _SP + n;
				acc = (t >> 8) | 1;
				aux = ((n ^ _SP ^ t) >> 8) ^ acc; nega = 0;
				SPhi = t & 0xF000; SPlo = t & 0x0FFF; stkpage = wmap[SPhi >> 12]; }
				tmp_clocks = 16;
				break;
		    case 0xF3: // DI
				cpu_ime = false;
				tmp_clocks = 4;
				break;
		    case 0xFB: // EI
				cpu_ime = int_disable = true;
				tmp_clocks = 4;
				break;
		    case 0x37: // SCF
				acc |= 0x100;
				aux = acc; nega = 0;
				tmp_clocks = 4;
				break;
		    case 0x3F: // CCF
				acc ^= 0x100;
				aux = acc; nega = 0;
				tmp_clocks = 4;
				break;
		    case 0xCB: // CB prefix
				throw new RuntimeException();
		    case 0x10: // STOP
				PClo += 1; 
				if ((R_KEY1 & 1) != 0) { cpu_speed ^= 1; R_KEY1 = (R_KEY1 & 0x7E) | (cpu_speed << 7); }
				tmp_clocks = 4;
				break;
		    case 0x76: // HALT
				default:
					cpu_halt = true;
					rest_clock = 0;
					break L1;
			// include end
		    }

		    rest_clock -= tmp_clocks;
		    //cpu_timers(tmp_clocks);
		}

		PC = PChi + PClo;
		SP = SPhi + SPlo;
		break L0;
	    }
	    catch (RuntimeException e) { // HIRAMłvÔŁBBB

		PC = PChi + lastPClo;
		SP = SPhi + SPlo;

		int op = mem_read(PC++), tmp_clocks = 0;

		switch (op) {
		    // include begin
		case 0x00: // NOP
		    tmp_clocks = 4;
		    break;
		case 0x40: // LD B,B
		    tmp_clocks = 4;
		    break;
		case 0x41: // LD B,C
		    B = C;
		    tmp_clocks = 4;
		    break;
		case 0x42: // LD B,D
		    B = D;
		    tmp_clocks = 4;
		    break;
		case 0x43: // LD B,E
		    B = E;
		    tmp_clocks = 4;
		    break;
		case 0x44: // LD B,H
		    B = ((HL >> 8) & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x45: // LD B,L
		    B = (HL & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x46: // LD B,(HL)
		    B = mem_read(HL);
		    tmp_clocks = 8;
		    break;
		case 0x47: // LD B,A
		    B = A;
		    tmp_clocks = 4;
		    break;
		case 0x48: // LD C,B
		    C = B;
		    tmp_clocks = 4;
		    break;
		case 0x49: // LD C,C
		    tmp_clocks = 4;
		    break;
		case 0x4A: // LD C,D
		    C = D;
		    tmp_clocks = 4;
		    break;
		case 0x4B: // LD C,E
		    C = E;
		    tmp_clocks = 4;
		    break;
		case 0x4C: // LD C,H
		    C = ((HL >> 8) & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x4D: // LD C,L
		    C = (HL & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x4E: // LD C,(HL)
		    C = mem_read(HL);
		    tmp_clocks = 8;
		    break;
		case 0x4F: // LD C,A
		    C = A;
		    tmp_clocks = 4;
		    break;
		case 0x50: // LD D,B
		    D = B;
		    tmp_clocks = 4;
		    break;
		case 0x51: // LD D,C
		    D = C;
		    tmp_clocks = 4;
		    break;
		case 0x52: // LD D,D
		    tmp_clocks = 4;
		    break;
		case 0x53: // LD D,E
		    D = E;
		    tmp_clocks = 4;
		    break;
		case 0x54: // LD D,H
		    D = ((HL >> 8) & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x55: // LD D,L
		    D = (HL & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x56: // LD D,(HL)
		    D = mem_read(HL);
		    tmp_clocks = 8;
		    break;
		case 0x57: // LD D,A
		    D = A;
		    tmp_clocks = 4;
		    break;
		case 0x58: // LD E,B
		    E = B;
		    tmp_clocks = 4;
		    break;
		case 0x59: // LD E,C
		    E = C;
		    tmp_clocks = 4;
		    break;
		case 0x5A: // LD E,D
		    E = D;
		    tmp_clocks = 4;
		    break;
		case 0x5B: // LD E,E
		    tmp_clocks = 4;
		    break;
		case 0x5C: // LD E,H
		    E = ((HL >> 8) & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x5D: // LD E,L
		    E = (HL & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x5E: // LD E,(HL)
		    E = mem_read(HL);
		    tmp_clocks = 8;
		    break;
		case 0x5F: // LD E,A
		    E = A;
		    tmp_clocks = 4;
		    break;
		case 0x60: // LD H,B
		    HL = ((B) << 8) | (HL & 0x00FF);
		    tmp_clocks = 4;
		    break;
		case 0x61: // LD H,C
		    HL = ((C) << 8) | (HL & 0x00FF);
		    tmp_clocks = 4;
		    break;
		case 0x62: // LD H,D
		    HL = ((D) << 8) | (HL & 0x00FF);
		    tmp_clocks = 4;
		    break;
		case 0x63: // LD H,E
		    HL = ((E) << 8) | (HL & 0x00FF);
		    tmp_clocks = 4;
		    break;
		case 0x64: // LD H,H
		    tmp_clocks = 4;
		    break;
		case 0x65: // LD H,L
		    HL = (((HL & 0xFF)) << 8) | (HL & 0x00FF);
		    tmp_clocks = 4;
		    break;
		case 0x66: // LD H,(HL)
		    HL = ((mem_read(HL)) << 8) | (HL & 0x00FF);
		    tmp_clocks = 8;
		    break;
		case 0x67: // LD H,A
		    HL = ((A) << 8) | (HL & 0x00FF);
		    tmp_clocks = 4;
		    break;
		case 0x68: // LD L,B
		    HL = (HL & 0xFF00) | (B);
		    tmp_clocks = 4;
		    break;
		case 0x69: // LD L,C
		    HL = (HL & 0xFF00) | (C);
		    tmp_clocks = 4;
		    break;
		case 0x6A: // LD L,D
		    HL = (HL & 0xFF00) | (D);
		    tmp_clocks = 4;
		    break;
		case 0x6B: // LD L,E
		    HL = (HL & 0xFF00) | (E);
		    tmp_clocks = 4;
		    break;
		case 0x6C: // LD L,H
		    HL = (HL & 0xFF00) | (((HL >> 8) & 0xFF));
		    tmp_clocks = 4;
		    break;
		case 0x6D: // LD L,L
		    tmp_clocks = 4;
		    break;
		case 0x6E: // LD L,(HL)
		    HL = (HL & 0xFF00) | (mem_read(HL));
		    tmp_clocks = 8;
		    break;
		case 0x6F: // LD L,A
		    HL = (HL & 0xFF00) | (A);
		    tmp_clocks = 4;
		    break;
		case 0x70: // LD (HL),B
		    mem_write(HL, B);
		    tmp_clocks = 8;
		    break;
		case 0x71: // LD (HL),C
		    mem_write(HL, C);
		    tmp_clocks = 8;
		    break;
		case 0x72: // LD (HL),D
		    mem_write(HL, D);
		    tmp_clocks = 8;
		    break;
		case 0x73: // LD (HL),E
		    mem_write(HL, E);
		    tmp_clocks = 8;
		    break;
		case 0x74: // LD (HL),H
		    mem_write(HL, ((HL >> 8) & 0xFF));
		    tmp_clocks = 8;
		    break;
		case 0x75: // LD (HL),L
		    mem_write(HL, (HL & 0xFF));
		    tmp_clocks = 8;
		    break;
		case 0x77: // LD (HL),A
		    mem_write(HL, A);
		    tmp_clocks = 8;
		    break;
		case 0x78: // LD A,B
		    A = B;
		    tmp_clocks = 4;
		    break;
		case 0x79: // LD A,C
		    A = C;
		    tmp_clocks = 4;
		    break;
		case 0x7A: // LD A,D
		    A = D;
		    tmp_clocks = 4;
		    break;
		case 0x7B: // LD A,E
		    A = E;
		    tmp_clocks = 4;
		    break;
		case 0x7C: // LD A,H
		    A = ((HL >> 8) & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x7D: // LD A,L
		    A = (HL & 0xFF);
		    tmp_clocks = 4;
		    break;
		case 0x7E: // LD A,(HL)
		    A = mem_read(HL);
		    tmp_clocks = 8;
		    break;
		case 0x7F: // LD A,A
		    tmp_clocks = 4;
		    break;
		case 0x01: // LD BC,imm
		    { int t = mem_readw(PC); PC += 2; B = (t >> 8) & 0xFF; C = t & 0xFF; }
		    tmp_clocks = 12;
		    break;
		case 0x11: // LD DE,imm
		    { int t = mem_readw(PC); PC += 2; D = (t >> 8) & 0xFF; E = t & 0xFF; }
		    tmp_clocks = 12;
		    break;
		case 0x21: // LD HL,imm
		    { int t = mem_readw(PC); PC += 2; HL = t;}
		    tmp_clocks = 12;
		    break;
		case 0x31: // LD SP,imm
		    { int t = mem_readw(PC); PC += 2; SP = t; }
		    tmp_clocks = 12;
		    break;
		case 0x02: // LD (BC),A
		    { int t = ((B << 8) | C);
		    mem_write(t, A);}
		    tmp_clocks = 8;
		    break;
		case 0x12: // LD (DE),A
		    { int t = ((D << 8) | E);
		    mem_write(t, A);}
		    tmp_clocks = 8;
		    break;
		case 0x22: // LDI (HL),A
		    mem_write(HL, A);HL++;
		    tmp_clocks = 8;
		    break;
		case 0x32: // LDD (HL),A
		    mem_write(HL, A);HL--;
		    tmp_clocks = 8;
		    break;
		case 0x0A: // LD A,(BC)
		    { int t = ((B << 8) | C);
		    A = mem_read(t); }
		    tmp_clocks = 8;
		    break;
		case 0x1A: // LD A,(DE)
		    { int t = ((D << 8) | E);
		    A = mem_read(t); }
		    tmp_clocks = 8;
		    break;
		case 0x2A: // LDI A,(HL)
		    A = mem_read(HL); HL++;
		    tmp_clocks = 8;
		    break;
		case 0x3A: // LDD A,(HL))
		    A = mem_read(HL); HL--;
		    tmp_clocks = 8;
		    break;
		case 0x06: // LD B,imm
		    { int t = mem_read(PC++); B = t;}
		    tmp_clocks = 8;
		    break;
		case 0x0E: // LD C,imm
		    { int t = mem_read(PC++); C = t;}
		    tmp_clocks = 8;
		    break;
		case 0x16: // LD D,imm
		    { int t = mem_read(PC++); D = t;}
		    tmp_clocks = 8;
		    break;
		case 0x1E: // LD E,imm
		    { int t = mem_read(PC++); E = t;}
		    tmp_clocks = 8;
		    break;
		case 0x26: // LD H,imm
		    { int t = mem_read(PC++); HL = ((t) << 8) | (HL & 0x00FF);}
		    tmp_clocks = 8;
		    break;
		case 0x2E: // LD L,imm
		    { int t = mem_read(PC++); HL = (HL & 0xFF00) | (t);}
		    tmp_clocks = 8;
		    break;
		case 0x36: // LD (HL),imm
		    { int t = mem_read(PC++); mem_write(HL, t);}
		    tmp_clocks = 12;
		    break;
		case 0x3E: // LD A,imm
		    { int t = mem_read(PC++); A = t;}
		    tmp_clocks = 8;
		    break;
		case 0x08: // LD (imm),SP
		    { int t = mem_readw(PC); PC += 2; mem_writew(t, SP); }
		    tmp_clocks = 20;
		    break;
		case 0xEA: // LD (imm),A
		    { int t = mem_readw(PC); PC += 2; mem_write(t, A);}
		    tmp_clocks = 16;
		    break;
		case 0xE0: // LDH (imm),A
		    { int t = mem_read(PC++); mem_writehi(t, A); }
		    tmp_clocks = 12;
		    break;
		case 0xE2: // LDH (C),A
		    mem_writehi(C, A);
		    tmp_clocks = 8;
		    break;
		case 0xF0: // LDH A,(imm)
		    { int t = mem_read(PC++); A = mem_readhi(t); }
		    tmp_clocks = 12;
		    break;
		case 0xF2: // LDH A,(C)
		    A = mem_readhi(C);
		    tmp_clocks = 8;
		    break;
		case 0xF8: // LD HL,SP+imm
		    { int n = (byte)mem_read(PC++); 
		    int _SP = SP; 
		    int t = _SP + n;
		    acc = (t >> 8) | 1;
		    aux = ((t ^ _SP ^ n) >> 8) ^ acc; nega = 0;
		    HL = t & 0xFFFF;}
		    tmp_clocks = 12;
		    break;
		case 0xF9: // LD SP,HL
		    SP = HL; 
		    tmp_clocks = 8;
		    break;
		case 0xFA: // LD A,(imm)
		    { int t = mem_readw(PC); PC += 2; A = mem_read(t); }
		    tmp_clocks = 16;
		    break;
		case 0xC6: // ADD A,imm
		    { int n = mem_read(PC++); 
		    acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x80: // ADD A,B
		    {
			acc = A + B; aux = A ^ B; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x81: // ADD A,C
		    {
			acc = A + C; aux = A ^ C; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x82: // ADD A,D
		    {
			acc = A + D; aux = A ^ D; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x83: // ADD A,E
		    {
			acc = A + E; aux = A ^ E; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x84: // ADD A,H
		    { int n = ((HL >> 8) & 0xFF);
		    acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x85: // ADD A,L
		    { int n = (HL & 0xFF);
		    acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x86: // ADD A,(HL)
		    { int n = mem_read(HL);
		    acc = A + n; aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x87: // ADD A,A
		    {
			acc = A + A; aux = A ^ A; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xCE: // ADC A,imm
		    { int n = mem_read(PC++); 
		    acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x88: // ADC A,B
		    {
			acc = A + B + ((acc >> 8) & 1); aux = A ^ B; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x89: // ADC A,C
		    {
			acc = A + C + ((acc >> 8) & 1); aux = A ^ C; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x8A: // ADC A,D
		    {
			acc = A + D + ((acc >> 8) & 1); aux = A ^ D; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x8B: // ADC A,E
		    {
			acc = A + E + ((acc >> 8) & 1); aux = A ^ E; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x8C: // ADC A,H
		    { int n = ((HL >> 8) & 0xFF);
		    acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x8D: // ADC A,L
		    { int n = (HL & 0xFF);
		    acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x8E: // ADC A,(HL)
		    { int n = mem_read(HL);
		    acc = A + n + ((acc >> 8) & 1); aux = A ^ n; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x8F: // ADC A,A
		    {
			acc = A + A + ((acc >> 8) & 1); aux = A ^ A; nega = 0; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xD6: // SUB imm
		    { int n = mem_read(PC++); 
		    acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x90: // SUB B
		    {
			acc = A - B; aux = A ^ B; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x91: // SUB C
		    {
			acc = A - C; aux = A ^ C; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x92: // SUB D
		    {
			acc = A - D; aux = A ^ D; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x93: // SUB E
		    {
			acc = A - E; aux = A ^ E; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x94: // SUB H
		    { int n = ((HL >> 8) & 0xFF);
		    acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x95: // SUB L
		    { int n = (HL & 0xFF);
		    acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x96: // SUB (HL)
		    { int n = mem_read(HL);
		    acc = A - n; aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x97: // SUB A
		    {
			acc = A - A; aux = A ^ A; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xDE: // SBC A,imm
		    { int n = mem_read(PC++); 
		    acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x98: // SBC A,B
		    {
			acc = A - B - ((acc >> 8) & 1); aux = A ^ B; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x99: // SBC A,C
		    {
			acc = A - C - ((acc >> 8) & 1); aux = A ^ C; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x9A: // SBC A,D
		    {
			acc = A - D - ((acc >> 8) & 1); aux = A ^ D; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x9B: // SBC A,E
		    {
			acc = A - E - ((acc >> 8) & 1); aux = A ^ E; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x9C: // SBC A,H
		    { int n = ((HL >> 8) & 0xFF);
		    acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x9D: // SBC A,L
		    { int n = (HL & 0xFF);
		    acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x9E: // SBC A,(HL)
		    { int n = mem_read(HL);
		    acc = A - n - ((acc >> 8) & 1); aux = A ^ n; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 8;
		    break;
		case 0x9F: // SBC A,A
		    {
			acc = A - A - ((acc >> 8) & 1); aux = A ^ A; nega = 1; A = acc & 0xFF;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xE6: // AND imm
		    { int n = mem_read(PC++); 
		    A = acc = A & n; aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xA0: // AND B
		    {
			A = acc = A & B; aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA1: // AND C
		    {
			A = acc = A & C; aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA2: // AND D
		    {
			A = acc = A & D; aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA3: // AND E
		    {
			A = acc = A & E; aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA4: // AND H
		    {
			A = acc = A & ((HL >> 8) & 0xFF); aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA5: // AND L
		    {
			A = acc = A & (HL & 0xFF); aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA6: // AND (HL)
		    {
			A = acc = A & mem_read(HL); aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xA7: // AND A
		    {
			A = acc = A & A; aux = ~acc; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xEE: // XOR imm
		    { int n = mem_read(PC++); 
		    A = aux = acc = A ^ n; nega = 0;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xA8: // XOR B
		    {
			A = aux = acc = A ^ B; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xA9: // XOR C
		    {
			A = aux = acc = A ^ C; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xAA: // XOR D
		    {
			A = aux = acc = A ^ D; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xAB: // XOR E
		    {
			A = aux = acc = A ^ E; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xAC: // XOR H
		    {
			A = aux = acc = A ^ ((HL >> 8) & 0xFF); nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xAD: // XOR L
		    {
			A = aux = acc = A ^ (HL & 0xFF); nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xAE: // XOR (HL)
		    {
			A = aux = acc = A ^ mem_read(HL); nega = 0;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xAF: // XOR A
		    {
			A = aux = acc = A ^ A; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xF6: // OR imm
		    { int n = mem_read(PC++); 
		    A = aux = acc = A | n; nega = 0;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xB0: // OR B
		    {
			A = aux = acc = A | B; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB1: // OR C
		    {
			A = aux = acc = A | C; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB2: // OR D
		    {
			A = aux = acc = A | D; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB3: // OR E
		    {
			A = aux = acc = A | E; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB4: // OR H
		    {
			A = aux = acc = A | ((HL >> 8) & 0xFF); nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB5: // OR L
		    {
			A = aux = acc = A | (HL & 0xFF); nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB6: // OR (HL)
		    {
			A = aux = acc = A | mem_read(HL); nega = 0;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xB7: // OR A
		    {
			A = aux = acc = A | A; nega = 0;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xFE: // CP imm
		    { int n = mem_read(PC++); 
		    acc = A - n; aux = A ^ n; nega = 1;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xB8: // CP B
		    {
			acc = A - B; aux = A ^ B; nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xB9: // CP C
		    {
			acc = A - C; aux = A ^ C; nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xBA: // CP D
		    {
			acc = A - D; aux = A ^ D; nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xBB: // CP E
		    {
			acc = A - E; aux = A ^ E; nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xBC: // CP H
		    {
			acc = A - ((HL >> 8) & 0xFF); aux = A ^ ((HL >> 8) & 0xFF); nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xBD: // CP L
		    {
			acc = A - (HL & 0xFF); aux = A ^ (HL & 0xFF); nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0xBE: // CP (HL)
		    {
			acc = A - mem_read(HL); aux = A ^ mem_read(HL); nega = 1;
		    }
		    tmp_clocks = 8;
		    break;
		case 0xBF: // CP A
		    {
			acc = A - A; aux = A ^ A; nega = 1;
		    }
		    tmp_clocks = 4;
		    break;
		case 0x09: // ADD HL,BC
		    { int n = ((B << 8) | C);
		    int t = HL + n;
		    acc = ((t >> 8) & 0x100) | (acc & 0xFF);
		    aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
		    HL = t & 0xFFFF;}
		    tmp_clocks = 8;
		    break;
		case 0x19: // ADD HL,DE
		    { int n = ((D << 8) | E);
		    int t = HL + n;
		    acc = ((t >> 8) & 0x100) | (acc & 0xFF);
		    aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
		    HL = t & 0xFFFF;}
		    tmp_clocks = 8;
		    break;
		case 0x29: // ADD HL,HL
		    { int n = HL;
		    int t = HL + n;
		    acc = ((t >> 8) & 0x100) | (acc & 0xFF);
		    aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
		    HL = t & 0xFFFF;}
		    tmp_clocks = 8;
		    break;
		case 0x39: // ADD HL,SP
		    { int n = SP;
		    int t = HL + n;
		    acc = ((t >> 8) & 0x100) | (acc & 0xFF);
		    aux = ((t ^ HL ^ n) >> 8) ^ acc; nega = 0;
		    HL = t & 0xFFFF;}
		    tmp_clocks = 8;
		    break;
		case 0x04: // INC B
		    aux = B; nega = 0; B = (B + 1) & 0xFF; acc = B | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x0C: // INC C
		    aux = C; nega = 0; C = (C + 1) & 0xFF; acc = C | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x14: // INC D
		    aux = D; nega = 0; D = (D + 1) & 0xFF; acc = D | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x1C: // INC E
		    aux = E; nega = 0; E = (E + 1) & 0xFF; acc = E | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x24: // INC H
		    { int n = ((HL >> 8) & 0xFF);
		    aux = n; nega = 0; n = (n + 1) & 0xFF; acc = n | (acc & 0x100);
		    HL = ((n) << 8) | (HL & 0x00FF);}
		    tmp_clocks = 4;
		    break;
		case 0x2C: // INC L
		    { int n = (HL & 0xFF);
		    aux = n; nega = 0; n = (n + 1) & 0xFF; acc = n | (acc & 0x100);
		    HL = (HL & 0xFF00) | (n);}
		    tmp_clocks = 4;
		    break;
		case 0x34: // INC (HL)
		    { int n = mem_read(HL);
		    aux = n; nega = 0; n = (n + 1) & 0xFF; acc = n | (acc & 0x100);
		    mem_write(HL, n);}
		    tmp_clocks = 12;
		    break;
		case 0x3C: // INC A
		    aux = A; nega = 0; A = (A + 1) & 0xFF; acc = A | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x05: // DEC B
		    aux = B; nega = 1; B = (B - 1) & 0xFF; acc = B | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x0D: // DEC C
		    aux = C; nega = 1; C = (C - 1) & 0xFF; acc = C | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x15: // DEC D
		    aux = D; nega = 1; D = (D - 1) & 0xFF; acc = D | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x1D: // DEC E
		    aux = E; nega = 1; E = (E - 1) & 0xFF; acc = E | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x25: // DEC H
		    { int n = ((HL >> 8) & 0xFF);
		    aux = n; nega = 1; n = (n - 1) & 0xFF; acc = n | (acc & 0x100);
		    HL = ((n) << 8) | (HL & 0x00FF);}
		    tmp_clocks = 4;
		    break;
		case 0x2D: // DEC L
		    { int n = (HL & 0xFF);
		    aux = n; nega = 1; n = (n - 1) & 0xFF; acc = n | (acc & 0x100);
		    HL = (HL & 0xFF00) | (n);}
		    tmp_clocks = 4;
		    break;
		case 0x35: // DEC (HL)
		    { int n = mem_read(HL);
		    aux = n; nega = 1; n = (n - 1) & 0xFF; acc = n | (acc & 0x100);
		    mem_write(HL, n);}
		    tmp_clocks = 12;
		    break;
		case 0x3D: // DEC A
		    aux = A; nega = 1; A = (A - 1) & 0xFF; acc = A | (acc & 0x100);
		    tmp_clocks = 4;
		    break;
		case 0x03: // INC BC
		    { int s = ((B << 8) | C) + 1; B = (s >> 8) & 0xFF; C = s & 0xFF; } 
		    tmp_clocks = 8;
		    break;
		case 0x13: // INC DE
		    { int s = ((D << 8) | E) + 1; D = (s >> 8) & 0xFF; E = s & 0xFF; } 
		    tmp_clocks = 8;
		    break;
		case 0x23: // INC HL
		    HL = (HL + 1) & 0xFFFF;
		    tmp_clocks = 8;
		    break;
		case 0x33: // INC SP
		    SP = SP + 1; 
		    tmp_clocks = 8;
		    break;
		case 0x0B: // DEC BC
		    { int s = ((B << 8) | C) - 1; B = (s >> 8) & 0xFF; C = s & 0xFF; } 
		    tmp_clocks = 8;
		    break;
		case 0x1B: // DEC DE
		    { int s = ((D << 8) | E) - 1; D = (s >> 8) & 0xFF; E = s & 0xFF; } 
		    tmp_clocks = 8;
		    break;
		case 0x2B: // DEC HL
		    HL = (HL - 1) & 0xFFFF;
		    tmp_clocks = 8;
		    break;
		case 0x3B: // DEC SP
		    SP = SP - 1; 
		    tmp_clocks = 8;
		    break;
		case 0x07: // RLCA
		    aux = acc = (A << 1) | (A >> 7); A = acc & 0xFF; nega = 0;
		    acc |= 1;
		    tmp_clocks = 4;
		    break;
		case 0x0F: // RRCA
		    aux = acc = ((A & 1) << 7) | (A >> 1) | (A << 8); A = acc & 0xFF; nega = 0;
		    acc |= 1;
		    tmp_clocks = 4;
		    break;
		case 0x17: // RLA
		    aux = acc = (A << 1) | ((acc >> 8) & 1); A = acc & 0xFF; nega = 0;
		    acc |= 1;
		    tmp_clocks = 4;
		    break;
		case 0x1F: // RRA
		    aux = acc = ((acc >> 1) & 0x80) | (A >> 1) | (A << 8); A = acc & 0xFF; nega = 0;
		    acc |= 1;
		    tmp_clocks = 4;
		    break;
		case 0x27: // DAA
		    { int n = daa(((packF(acc, aux, nega) & 0x70) << 4) | A);
		    A = (A + n) & 0xFF;
		    acc = (daa_carry(n >> 2) << 4) | A; aux = ~acc; }
		    tmp_clocks = 4;
		    break;
		case 0x2F: // CPL
		    A = ~A & 0xFF;
		    aux = ~acc; nega = 1;
		    tmp_clocks = 4;
		    break;
		case 0x18: // JR
		    { int t = (byte)mem_read(PC++); PC += t; }
		    tmp_clocks = 12;
		    break;
		case 0x20: // JR NZ
		    if ((acc & 0xFF) != 0) {
			int t = (byte)mem_read(PC++); 
			PC += t; 
			tmp_clocks = 12;
			break;
		    } else {
			PC += 1; 
			tmp_clocks = 8;
			break;
		    }
		case 0x28: // JR Z
		    if ((acc & 0xFF) == 0) {
			int t = (byte)mem_read(PC++); 
			PC += t; 
			tmp_clocks = 12;
			break;
		    } else {
			PC += 1; 
			tmp_clocks = 8;
			break;
		    }
		case 0x30: // JR NC
		    if ((acc & 0x100) == 0) {
			int t = (byte)mem_read(PC++); 
			PC += t; 
			tmp_clocks = 12;
			break;
		    } else {
			PC += 1; 
			tmp_clocks = 8;
			break;
		    }
		case 0x38: // JR C
		    if ((acc & 0x100) != 0) {
			int t = (byte)mem_read(PC++); 
			PC += t; 
			tmp_clocks = 12;
			break;
		    } else {
			PC += 1; 
			tmp_clocks = 8;
			break;
		    }
		case 0xC3: // JP
		    { int t = mem_readw(PC); PC += 2; PC = t; }
		    tmp_clocks = 16;
		    break;
		case 0xC2: // JP NZ
		    if ((acc & 0xFF) != 0) {
			int t = mem_readw(PC); PC += 2; 
			PC = t; 
			tmp_clocks = 16;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xCA: // JP Z
		    if ((acc & 0xFF) == 0) {
			int t = mem_readw(PC); PC += 2; 
			PC = t; 
			tmp_clocks = 16;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xD2: // JP NC
		    if ((acc & 0x100) == 0) {
			int t = mem_readw(PC); PC += 2; 
			PC = t; 
			tmp_clocks = 16;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xDA: // JP C
		    if ((acc & 0x100) != 0) {
			int t = mem_readw(PC); PC += 2; 
			PC = t; 
			tmp_clocks = 16;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xE9: // JP (HL)
		    PC = HL; 
		    tmp_clocks = 4;
		    break;
		case 0xC9: // RET
		    { int t = mem_readw(SP); SP += 2; PC = t; }
		    tmp_clocks = 16;
		    break;
		case 0xC0: // RET NZ
		    if ((acc & 0xFF) != 0) {
			int t = mem_readw(SP); SP += 2; 
			PC = t; 
			tmp_clocks = 20;
			break;
		    } else {
			tmp_clocks = 8;
			break;
		    }
		case 0xC8: // RET Z
		    if ((acc & 0xFF) == 0) {
			int t = mem_readw(SP); SP += 2; 
			PC = t; 
			tmp_clocks = 20;
			break;
		    } else {
			tmp_clocks = 8;
			break;
		    }
		case 0xD0: // RET NC
		    if ((acc & 0x100) == 0) {
			int t = mem_readw(SP); SP += 2; 
			PC = t; 
			tmp_clocks = 20;
			break;
		    } else {
			tmp_clocks = 8;
			break;
		    }
		case 0xD8: // RET C
		    if ((acc & 0x100) != 0) {
			int t = mem_readw(SP); SP += 2; 
			PC = t; 
			tmp_clocks = 20;
			break;
		    } else {
			tmp_clocks = 8;
			break;
		    }
		case 0xD9: // RETI
		    cpu_ime = int_disable = true;
		    { int t = mem_readw(SP); SP += 2; PC = t; }
		    tmp_clocks = 16;
		    break;
		case 0xCD: // CALL
		    { int t = mem_readw(PC); PC += 2; SP -= 2; mem_writew(SP, PC); PC = t; }
		    tmp_clocks = 24;
		    break;
		case 0xC4: // CALL NZ
		    if ((acc & 0xFF) != 0) {
			int t = mem_readw(PC); PC += 2; 
			SP -= 2; mem_writew(SP, PC); 
			PC = t; 
			tmp_clocks = 24;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xCC: // CALL Z
		    if ((acc & 0xFF) == 0) {
			int t = mem_readw(PC); PC += 2; 
			SP -= 2; mem_writew(SP, PC); 
			PC = t; 
			tmp_clocks = 24;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xD4: // CALL NC
		    if ((acc & 0x100) == 0) {
			int t = mem_readw(PC); PC += 2; 
			SP -= 2; mem_writew(SP, PC); 
			PC = t; 
			tmp_clocks = 24;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xDC: // CALL C
		    if ((acc & 0x100) != 0) {
			int t = mem_readw(PC); PC += 2; 
			SP -= 2; mem_writew(SP, PC); 
			PC = t; 
			tmp_clocks = 24;
			break;
		    } else {
			PC += 2; 
			tmp_clocks = 12;
			break;
		    }
		case 0xC7: // RST 0
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 0; 
		    tmp_clocks = 16;
		    break;
		case 0xCF: // RST 8
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 8; 
		    tmp_clocks = 16;
		    break;
		case 0xD7: // RST 10
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 16; 
		    tmp_clocks = 16;
		    break;
		case 0xDF: // RST 18
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 24; 
		    tmp_clocks = 16;
		    break;
		case 0xE7: // RST 20
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 32; 
		    tmp_clocks = 16;
		    break;
		case 0xEF: // RST 28
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 40; 
		    tmp_clocks = 16;
		    break;
		case 0xF7: // RST 30
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 48; 
		    tmp_clocks = 16;
		    break;
		case 0xFF: // RST 38
		    SP -= 2; mem_writew(SP, PC); 
		    PC = 56; 
		    tmp_clocks = 16;
		    break;
		case 0xC1: // POP BC
		    { int t = mem_readw(SP); SP += 2; B = (t >> 8) & 0xFF; C = t & 0xFF; }
		    tmp_clocks = 12;
		    break;
		case 0xD1: // POP DE
		    { int t = mem_readw(SP); SP += 2; D = (t >> 8) & 0xFF; E = t & 0xFF; }
		    tmp_clocks = 12;
		    break;
		case 0xE1: // POP HL
		    HL = mem_readw(SP); SP += 2; 
		    tmp_clocks = 12;
		    break;
		case 0xF1: // POP AF
		    { int t = mem_readw(SP); SP += 2; 
		    A = (t >> 8) & 0xFF;
		    acc = (~t & 0x80) | ((t & 0x10) << 4);
		    aux = t >> 1;  nega = (t >> 6) & 1; }
		    tmp_clocks = 12;
		    break;
		case 0xC5: // PUSH BC
		    SP -= 2; mem_write(SP, C); mem_write(SP + 1, B); 
		    tmp_clocks = 16;
		    break;
		case 0xD5: // PUSH DE
		    SP -= 2; mem_write(SP, E); mem_write(SP + 1, D); 
		    tmp_clocks = 16;
		    break;
		case 0xE5: // PUSH HL
		    SP -= 2; mem_writew(SP, HL); 
		    tmp_clocks = 16;
		    break;
		case 0xF5: // PUSH AF
		    SP -= 2; mem_write(SP, (packF(acc, aux, nega) | 0x0E)); mem_write(SP + 1, A); 
		    tmp_clocks = 16;
		    break;
		case 0xE8: // ADD SP,imm
		    { int n = (byte)mem_read(PC++); 
		    int _SP = SP; 
		    int t = _SP + n;
		    acc = (t >> 8) | 1;
		    aux = ((n ^ _SP ^ t) >> 8) ^ acc; nega = 0;
		    SP = t; }
		    tmp_clocks = 16;
		    break;
		case 0xF3: // DI
		    cpu_ime = false;
		    tmp_clocks = 4;
		    break;
		case 0xFB: // EI
		    cpu_ime = int_disable = true;
		    tmp_clocks = 4;
		    break;
		case 0x37: // SCF
		    acc |= 0x100;
		    aux = acc; nega = 0;
		    tmp_clocks = 4;
		    break;
		case 0x3F: // CCF
		    acc ^= 0x100;
		    aux = acc; nega = 0;
		    tmp_clocks = 4;
		    break;
		case 0xCB: // CB prefix
		    { int cbop = mem_read(PC++); 
		    switch (cbop) {
		    case 0x00: // RLC B
			aux = acc = (B << 1) | (B >> 7); B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x01: // RLC C
			aux = acc = (C << 1) | (C >> 7); C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x02: // RLC D
			aux = acc = (D << 1) | (D >> 7); D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x03: // RLC E
			aux = acc = (E << 1) | (E >> 7); E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x04: // RLC H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = (n << 1) | (n >> 7); n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x05: // RLC L
			{ int n = (HL & 0xFF);
			aux = acc = (n << 1) | (n >> 7); n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x06: // RLC (HL)
			{ int n = mem_read(HL);
			aux = acc = (n << 1) | (n >> 7); n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x07: // RLC A
			aux = acc = (A << 1) | (A >> 7); A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x08: // RRC B
			aux = acc = ((B & 1) << 7) | (B >> 1) | (B << 8); B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x09: // RRC C
			aux = acc = ((C & 1) << 7) | (C >> 1) | (C << 8); C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x0A: // RRC D
			aux = acc = ((D & 1) << 7) | (D >> 1) | (D << 8); D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x0B: // RRC E
			aux = acc = ((E & 1) << 7) | (E >> 1) | (E << 8); E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x0C: // RRC H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = ((n & 1) << 7) | (n >> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x0D: // RRC L
			{ int n = (HL & 0xFF);
			aux = acc = ((n & 1) << 7) | (n >> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x0E: // RRC (HL)
			{ int n = mem_read(HL);
			aux = acc = ((n & 1) << 7) | (n >> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x0F: // RRC A
			aux = acc = ((A & 1) << 7) | (A >> 1) | (A << 8); A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x10: // RL B
			aux = acc = (B << 1) | ((acc >> 8) & 1); B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x11: // RL C
			aux = acc = (C << 1) | ((acc >> 8) & 1); C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x12: // RL D
			aux = acc = (D << 1) | ((acc >> 8) & 1); D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x13: // RL E
			aux = acc = (E << 1) | ((acc >> 8) & 1); E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x14: // RL H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = (n << 1) | ((acc >> 8) & 1); n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x15: // RL L
			{ int n = (HL & 0xFF);
			aux = acc = (n << 1) | ((acc >> 8) & 1); n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x16: // RL (HL)
			{ int n = mem_read(HL);
			aux = acc = (n << 1) | ((acc >> 8) & 1); n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x17: // RL A
			aux = acc = (A << 1) | ((acc >> 8) & 1); A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x18: // RR B
			aux = acc = ((acc >> 1) & 0x80) | (B >> 1) | (B << 8); B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x19: // RR C
			aux = acc = ((acc >> 1) & 0x80) | (C >> 1) | (C << 8); C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x1A: // RR D
			aux = acc = ((acc >> 1) & 0x80) | (D >> 1) | (D << 8); D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x1B: // RR E
			aux = acc = ((acc >> 1) & 0x80) | (E >> 1) | (E << 8); E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x1C: // RR H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = ((acc >> 1) & 0x80) | (n >> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x1D: // RR L
			{ int n = (HL & 0xFF);
			aux = acc = ((acc >> 1) & 0x80) | (n >> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x1E: // RR (HL)
			{ int n = mem_read(HL);
			aux = acc = ((acc >> 1) & 0x80) | (n >> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x1F: // RR A
			aux = acc = ((acc >> 1) & 0x80) | (A >> 1) | (A << 8); A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x20: // SLA B
			aux = acc = B << 1; B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x21: // SLA C
			aux = acc = C << 1; C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x22: // SLA D
			aux = acc = D << 1; D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x23: // SLA E
			aux = acc = E << 1; E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x24: // SLA H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = n << 1; n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x25: // SLA L
			{ int n = (HL & 0xFF);
			aux = acc = n << 1; n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x26: // SLA (HL)
			{ int n = mem_read(HL);
			aux = acc = n << 1; n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x27: // SLA A
			aux = acc = A << 1; A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x28: // SRA B
			aux = acc = (((byte)B >> 1) & 0xFF) | (B << 8); B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x29: // SRA C
			aux = acc = (((byte)C >> 1) & 0xFF) | (C << 8); C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x2A: // SRA D
			aux = acc = (((byte)D >> 1) & 0xFF) | (D << 8); D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x2B: // SRA E
			aux = acc = (((byte)E >> 1) & 0xFF) | (E << 8); E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x2C: // SRA H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = (((byte)n >> 1) & 0xFF) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x2D: // SRA L
			{ int n = (HL & 0xFF);
			aux = acc = (((byte)n >> 1) & 0xFF) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x2E: // SRA (HL)
			{ int n = mem_read(HL);
			aux = acc = (((byte)n >> 1) & 0xFF) | (n << 8); n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x2F: // SRA A
			aux = acc = (((byte)A >> 1) & 0xFF) | (A << 8); A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x30: // SWAP B
			B = aux = acc = ((B & 0x0F) << 4) | ((B >> 4) & 0x0F); nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x31: // SWAP C
			C = aux = acc = ((C & 0x0F) << 4) | ((C >> 4) & 0x0F); nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x32: // SWAP D
			D = aux = acc = ((D & 0x0F) << 4) | ((D >> 4) & 0x0F); nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x33: // SWAP E
			E = aux = acc = ((E & 0x0F) << 4) | ((E >> 4) & 0x0F); nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x34: // SWAP H
			{ int n = ((HL >> 8) & 0xFF);
			n = aux = acc = ((n & 0x0F) << 4) | ((n >> 4) & 0x0F); nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x35: // SWAP L
			{ int n = (HL & 0xFF);
			n = aux = acc = ((n & 0x0F) << 4) | ((n >> 4) & 0x0F); nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x36: // SWAP (HL)
			{ int n = mem_read(HL);
			n = aux = acc = ((n & 0x0F) << 4) | ((n >> 4) & 0x0F); nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x37: // SWAP A
			A = aux = acc = ((A & 0x0F) << 4) | ((A >> 4) & 0x0F); nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x38: // SRA B
			aux = acc = (B >>> 1) | (B << 8); B = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x39: // SRA C
			aux = acc = (C >>> 1) | (C << 8); C = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x3A: // SRA D
			aux = acc = (D >>> 1) | (D << 8); D = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x3B: // SRA E
			aux = acc = (E >>> 1) | (E << 8); E = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x3C: // SRA H
			{ int n = ((HL >> 8) & 0xFF);
			aux = acc = (n >>> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = ((n) << 8) | (HL & 0x00FF);}
			tmp_clocks = 8;
			break;
		    case 0x3D: // SRA L
			{ int n = (HL & 0xFF);
			aux = acc = (n >>> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			HL = (HL & 0xFF00) | (n);}
			tmp_clocks = 8;
			break;
		    case 0x3E: // SRA (HL)
			{ int n = mem_read(HL);
			aux = acc = (n >>> 1) | (n << 8); n = acc & 0xFF; nega = 0;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x3F: // SRA A
			aux = acc = (A >>> 1) | (A << 8); A = acc & 0xFF; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x40: // BIT 0,B
			acc = (B & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x48: // BIT 1,B
			acc = (B & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x50: // BIT 2,B
			acc = (B & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x58: // BIT 3,B
			acc = (B & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x60: // BIT 4,B
			acc = (B & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x68: // BIT 5,B
			acc = (B & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x70: // BIT 6,B
			acc = (B & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x78: // BIT 7,B
			acc = (B & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x41: // BIT 0,C
			acc = (C & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x49: // BIT 1,C
			acc = (C & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x51: // BIT 2,C
			acc = (C & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x59: // BIT 3,C
			acc = (C & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x61: // BIT 4,C
			acc = (C & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x69: // BIT 5,C
			acc = (C & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x71: // BIT 6,C
			acc = (C & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x79: // BIT 7,C
			acc = (C & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x42: // BIT 0,D
			acc = (D & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x4A: // BIT 1,D
			acc = (D & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x52: // BIT 2,D
			acc = (D & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x5A: // BIT 3,D
			acc = (D & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x62: // BIT 4,D
			acc = (D & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x6A: // BIT 5,D
			acc = (D & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x72: // BIT 6,D
			acc = (D & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x7A: // BIT 7,D
			acc = (D & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x43: // BIT 0,E
			acc = (E & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x4B: // BIT 1,E
			acc = (E & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x53: // BIT 2,E
			acc = (E & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x5B: // BIT 3,E
			acc = (E & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x63: // BIT 4,E
			acc = (E & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x6B: // BIT 5,E
			acc = (E & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x73: // BIT 6,E
			acc = (E & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x7B: // BIT 7,E
			acc = (E & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x44: // BIT 0,H
			acc = ((HL >> 8) & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x4C: // BIT 1,H
			acc = ((HL >> 8) & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x54: // BIT 2,H
			acc = ((HL >> 8) & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x5C: // BIT 3,H
			acc = ((HL >> 8) & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x64: // BIT 4,H
			acc = ((HL >> 8) & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x6C: // BIT 5,H
			acc = ((HL >> 8) & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x74: // BIT 6,H
			acc = ((HL >> 8) & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x7C: // BIT 7,H
			acc = ((HL >> 8) & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x45: // BIT 0,L
			acc = (HL & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x4D: // BIT 1,L
			acc = (HL & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x55: // BIT 2,L
			acc = (HL & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x5D: // BIT 3,L
			acc = (HL & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x65: // BIT 4,L
			acc = (HL & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x6D: // BIT 5,L
			acc = (HL & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x75: // BIT 6,L
			acc = (HL & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x7D: // BIT 7,L
			acc = (HL & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x46: // BIT 0,(HL)
			{ int n = mem_read(HL);
			acc = (n & 1) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x4E: // BIT 1,(HL)
			{ int n = mem_read(HL);
			acc = (n & 2) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x56: // BIT 2,(HL)
			{ int n = mem_read(HL);
			acc = (n & 4) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x5E: // BIT 3,(HL)
			{ int n = mem_read(HL);
			acc = (n & 8) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x66: // BIT 4,(HL)
			{ int n = mem_read(HL);
			acc = (n & 16) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x6E: // BIT 5,(HL)
			{ int n = mem_read(HL);
			acc = (n & 32) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x76: // BIT 6,(HL)
			{ int n = mem_read(HL);
			acc = (n & 64) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x7E: // BIT 7,(HL)
			{ int n = mem_read(HL);
			acc = (n & 128) | (acc & 0x100); aux = ~acc; nega = 0;}
			tmp_clocks = 12;
			break;
		    case 0x47: // BIT 0,A
			acc = (A & 1) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x4F: // BIT 1,A
			acc = (A & 2) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x57: // BIT 2,A
			acc = (A & 4) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x5F: // BIT 3,A
			acc = (A & 8) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x67: // BIT 4,A
			acc = (A & 16) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x6F: // BIT 5,A
			acc = (A & 32) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x77: // BIT 6,A
			acc = (A & 64) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x7F: // BIT 7,A
			acc = (A & 128) | (acc & 0x100); aux = ~acc; nega = 0;
			tmp_clocks = 8;
			break;
		    case 0x80: // RES 0,B
			B &= ~1;
			tmp_clocks = 8;
			break;
		    case 0x88: // RES 1,B
			B &= ~2;
			tmp_clocks = 8;
			break;
		    case 0x90: // RES 2,B
			B &= ~4;
			tmp_clocks = 8;
			break;
		    case 0x98: // RES 3,B
			B &= ~8;
			tmp_clocks = 8;
			break;
		    case 0xA0: // RES 4,B
			B &= ~16;
			tmp_clocks = 8;
			break;
		    case 0xA8: // RES 5,B
			B &= ~32;
			tmp_clocks = 8;
			break;
		    case 0xB0: // RES 6,B
			B &= ~64;
			tmp_clocks = 8;
			break;
		    case 0xB8: // RES 7,B
			B &= ~128;
			tmp_clocks = 8;
			break;
		    case 0x81: // RES 0,C
			C &= ~1;
			tmp_clocks = 8;
			break;
		    case 0x89: // RES 1,C
			C &= ~2;
			tmp_clocks = 8;
			break;
		    case 0x91: // RES 2,C
			C &= ~4;
			tmp_clocks = 8;
			break;
		    case 0x99: // RES 3,C
			C &= ~8;
			tmp_clocks = 8;
			break;
		    case 0xA1: // RES 4,C
			C &= ~16;
			tmp_clocks = 8;
			break;
		    case 0xA9: // RES 5,C
			C &= ~32;
			tmp_clocks = 8;
			break;
		    case 0xB1: // RES 6,C
			C &= ~64;
			tmp_clocks = 8;
			break;
		    case 0xB9: // RES 7,C
			C &= ~128;
			tmp_clocks = 8;
			break;
		    case 0x82: // RES 0,D
			D &= ~1;
			tmp_clocks = 8;
			break;
		    case 0x8A: // RES 1,D
			D &= ~2;
			tmp_clocks = 8;
			break;
		    case 0x92: // RES 2,D
			D &= ~4;
			tmp_clocks = 8;
			break;
		    case 0x9A: // RES 3,D
			D &= ~8;
			tmp_clocks = 8;
			break;
		    case 0xA2: // RES 4,D
			D &= ~16;
			tmp_clocks = 8;
			break;
		    case 0xAA: // RES 5,D
			D &= ~32;
			tmp_clocks = 8;
			break;
		    case 0xB2: // RES 6,D
			D &= ~64;
			tmp_clocks = 8;
			break;
		    case 0xBA: // RES 7,D
			D &= ~128;
			tmp_clocks = 8;
			break;
		    case 0x83: // RES 0,E
			E &= ~1;
			tmp_clocks = 8;
			break;
		    case 0x8B: // RES 1,E
			E &= ~2;
			tmp_clocks = 8;
			break;
		    case 0x93: // RES 2,E
			E &= ~4;
			tmp_clocks = 8;
			break;
		    case 0x9B: // RES 3,E
			E &= ~8;
			tmp_clocks = 8;
			break;
		    case 0xA3: // RES 4,E
			E &= ~16;
			tmp_clocks = 8;
			break;
		    case 0xAB: // RES 5,E
			E &= ~32;
			tmp_clocks = 8;
			break;
		    case 0xB3: // RES 6,E
			E &= ~64;
			tmp_clocks = 8;
			break;
		    case 0xBB: // RES 7,E
			E &= ~128;
			tmp_clocks = 8;
			break;
		    case 0x84: // RES 0,H
			HL &= ~256;
			tmp_clocks = 8;
			break;
		    case 0x8C: // RES 1,H
			HL &= ~512;
			tmp_clocks = 8;
			break;
		    case 0x94: // RES 2,H
			HL &= ~1024;
			tmp_clocks = 8;
			break;
		    case 0x9C: // RES 3,H
			HL &= ~2048;
			tmp_clocks = 8;
			break;
		    case 0xA4: // RES 4,H
			HL &= ~4096;
			tmp_clocks = 8;
			break;
		    case 0xAC: // RES 5,H
			HL &= ~8192;
			tmp_clocks = 8;
			break;
		    case 0xB4: // RES 6,H
			HL &= ~16384;
			tmp_clocks = 8;
			break;
		    case 0xBC: // RES 7,H
			HL &= ~32768;
			tmp_clocks = 8;
			break;
		    case 0x85: // RES 0,L
			HL &= ~1;
			tmp_clocks = 8;
			break;
		    case 0x8D: // RES 1,L
			HL &= ~2;
			tmp_clocks = 8;
			break;
		    case 0x95: // RES 2,L
			HL &= ~4;
			tmp_clocks = 8;
			break;
		    case 0x9D: // RES 3,L
			HL &= ~8;
			tmp_clocks = 8;
			break;
		    case 0xA5: // RES 4,L
			HL &= ~16;
			tmp_clocks = 8;
			break;
		    case 0xAD: // RES 5,L
			HL &= ~32;
			tmp_clocks = 8;
			break;
		    case 0xB5: // RES 6,L
			HL &= ~64;
			tmp_clocks = 8;
			break;
		    case 0xBD: // RES 7,L
			HL &= ~128;
			tmp_clocks = 8;
			break;
		    case 0x86: // RES 0,(HL)
			{ int n = mem_read(HL);
			n &= ~1;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x8E: // RES 1,(HL)
			{ int n = mem_read(HL);
			n &= ~2;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x96: // RES 2,(HL)
			{ int n = mem_read(HL);
			n &= ~4;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x9E: // RES 3,(HL)
			{ int n = mem_read(HL);
			n &= ~8;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xA6: // RES 4,(HL)
			{ int n = mem_read(HL);
			n &= ~16;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xAE: // RES 5,(HL)
			{ int n = mem_read(HL);
			n &= ~32;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xB6: // RES 6,(HL)
			{ int n = mem_read(HL);
			n &= ~64;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xBE: // RES 7,(HL)
			{ int n = mem_read(HL);
			n &= ~128;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0x87: // RES 0,A
			A &= ~1;
			tmp_clocks = 8;
			break;
		    case 0x8F: // RES 1,A
			A &= ~2;
			tmp_clocks = 8;
			break;
		    case 0x97: // RES 2,A
			A &= ~4;
			tmp_clocks = 8;
			break;
		    case 0x9F: // RES 3,A
			A &= ~8;
			tmp_clocks = 8;
			break;
		    case 0xA7: // RES 4,A
			A &= ~16;
			tmp_clocks = 8;
			break;
		    case 0xAF: // RES 5,A
			A &= ~32;
			tmp_clocks = 8;
			break;
		    case 0xB7: // RES 6,A
			A &= ~64;
			tmp_clocks = 8;
			break;
		    case 0xBF: // RES 7,A
			A &= ~128;
			tmp_clocks = 8;
			break;
		    case 0xC0: // SET 0,B
			B |= 1;
			tmp_clocks = 8;
			break;
		    case 0xC8: // SET 1,B
			B |= 2;
			tmp_clocks = 8;
			break;
		    case 0xD0: // SET 2,B
			B |= 4;
			tmp_clocks = 8;
			break;
		    case 0xD8: // SET 3,B
			B |= 8;
			tmp_clocks = 8;
			break;
		    case 0xE0: // SET 4,B
			B |= 16;
			tmp_clocks = 8;
			break;
		    case 0xE8: // SET 5,B
			B |= 32;
			tmp_clocks = 8;
			break;
		    case 0xF0: // SET 6,B
			B |= 64;
			tmp_clocks = 8;
			break;
		    case 0xF8: // SET 7,B
			B |= 128;
			tmp_clocks = 8;
			break;
		    case 0xC1: // SET 0,C
			C |= 1;
			tmp_clocks = 8;
			break;
		    case 0xC9: // SET 1,C
			C |= 2;
			tmp_clocks = 8;
			break;
		    case 0xD1: // SET 2,C
			C |= 4;
			tmp_clocks = 8;
			break;
		    case 0xD9: // SET 3,C
			C |= 8;
			tmp_clocks = 8;
			break;
		    case 0xE1: // SET 4,C
			C |= 16;
			tmp_clocks = 8;
			break;
		    case 0xE9: // SET 5,C
			C |= 32;
			tmp_clocks = 8;
			break;
		    case 0xF1: // SET 6,C
			C |= 64;
			tmp_clocks = 8;
			break;
		    case 0xF9: // SET 7,C
			C |= 128;
			tmp_clocks = 8;
			break;
		    case 0xC2: // SET 0,D
			D |= 1;
			tmp_clocks = 8;
			break;
		    case 0xCA: // SET 1,D
			D |= 2;
			tmp_clocks = 8;
			break;
		    case 0xD2: // SET 2,D
			D |= 4;
			tmp_clocks = 8;
			break;
		    case 0xDA: // SET 3,D
			D |= 8;
			tmp_clocks = 8;
			break;
		    case 0xE2: // SET 4,D
			D |= 16;
			tmp_clocks = 8;
			break;
		    case 0xEA: // SET 5,D
			D |= 32;
			tmp_clocks = 8;
			break;
		    case 0xF2: // SET 6,D
			D |= 64;
			tmp_clocks = 8;
			break;
		    case 0xFA: // SET 7,D
			D |= 128;
			tmp_clocks = 8;
			break;
		    case 0xC3: // SET 0,E
			E |= 1;
			tmp_clocks = 8;
			break;
		    case 0xCB: // SET 1,E
			E |= 2;
			tmp_clocks = 8;
			break;
		    case 0xD3: // SET 2,E
			E |= 4;
			tmp_clocks = 8;
			break;
		    case 0xDB: // SET 3,E
			E |= 8;
			tmp_clocks = 8;
			break;
		    case 0xE3: // SET 4,E
			E |= 16;
			tmp_clocks = 8;
			break;
		    case 0xEB: // SET 5,E
			E |= 32;
			tmp_clocks = 8;
			break;
		    case 0xF3: // SET 6,E
			E |= 64;
			tmp_clocks = 8;
			break;
		    case 0xFB: // SET 7,E
			E |= 128;
			tmp_clocks = 8;
			break;
		    case 0xC4: // SET 0,H
			HL |= 256;
			tmp_clocks = 8;
			break;
		    case 0xCC: // SET 1,H
			HL |= 512;
			tmp_clocks = 8;
			break;
		    case 0xD4: // SET 2,H
			HL |= 1024;
			tmp_clocks = 8;
			break;
		    case 0xDC: // SET 3,H
			HL |= 2048;
			tmp_clocks = 8;
			break;
		    case 0xE4: // SET 4,H
			HL |= 4096;
			tmp_clocks = 8;
			break;
		    case 0xEC: // SET 5,H
			HL |= 8192;
			tmp_clocks = 8;
			break;
		    case 0xF4: // SET 6,H
			HL |= 16384;
			tmp_clocks = 8;
			break;
		    case 0xFC: // SET 7,H
			HL |= 32768;
			tmp_clocks = 8;
			break;
		    case 0xC5: // SET 0,L
			HL |= 1;
			tmp_clocks = 8;
			break;
		    case 0xCD: // SET 1,L
			HL |= 2;
			tmp_clocks = 8;
			break;
		    case 0xD5: // SET 2,L
			HL |= 4;
			tmp_clocks = 8;
			break;
		    case 0xDD: // SET 3,L
			HL |= 8;
			tmp_clocks = 8;
			break;
		    case 0xE5: // SET 4,L
			HL |= 16;
			tmp_clocks = 8;
			break;
		    case 0xED: // SET 5,L
			HL |= 32;
			tmp_clocks = 8;
			break;
		    case 0xF5: // SET 6,L
			HL |= 64;
			tmp_clocks = 8;
			break;
		    case 0xFD: // SET 7,L
			HL |= 128;
			tmp_clocks = 8;
			break;
		    case 0xC6: // SET 0,(HL)
			{ int n = mem_read(HL);
			n |= 1;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xCE: // SET 1,(HL)
			{ int n = mem_read(HL);
			n |= 2;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xD6: // SET 2,(HL)
			{ int n = mem_read(HL);
			n |= 4;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xDE: // SET 3,(HL)
			{ int n = mem_read(HL);
			n |= 8;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xE6: // SET 4,(HL)
			{ int n = mem_read(HL);
			n |= 16;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xEE: // SET 5,(HL)
			{ int n = mem_read(HL);
			n |= 32;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xF6: // SET 6,(HL)
			{ int n = mem_read(HL);
			n |= 64;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xFE: // SET 7,(HL)
			{ int n = mem_read(HL);
			n |= 128;
			mem_write(HL, n);}
			tmp_clocks = 16;
			break;
		    case 0xC7: // SET 0,A
			A |= 1;
			tmp_clocks = 8;
			break;
		    case 0xCF: // SET 1,A
			A |= 2;
			tmp_clocks = 8;
			break;
		    case 0xD7: // SET 2,A
			A |= 4;
			tmp_clocks = 8;
			break;
		    case 0xDF: // SET 3,A
			A |= 8;
			tmp_clocks = 8;
			break;
		    case 0xE7: // SET 4,A
			A |= 16;
			tmp_clocks = 8;
			break;
		    case 0xEF: // SET 5,A
			A |= 32;
			tmp_clocks = 8;
			break;
		    case 0xF7: // SET 6,A
			A |= 64;
			tmp_clocks = 8;
			break;
		    case 0xFF: // SET 7,A
			A |= 128;
			tmp_clocks = 8;
			break;
		    } }
		    break;
		case 0x10: // STOP
		    PC += 1; 
		    if ((R_KEY1 & 1) != 0) { cpu_speed ^= 1; R_KEY1 = (R_KEY1 & 0x7E) | (cpu_speed << 7); }
		    tmp_clocks = 4;
		    break;
		case 0x76: // HALT
		default:
		    cpu_halt = true;
		    rest_clock = 0;
		    break L0;
		    // include end
		}
		
		rest_clock -= tmp_clocks;
		//cpu_timers(tmp_clocks);

		continue L0;
            }
	}

	this.A = A;
	F = packF(acc, aux, nega);
	this.B = B;
	this.C = C;
	this.D = D;
	this.E = E;
	this.HL = HL;
	this.SP = SP;

	return rest_clock_ - rest_clock;
    }

    private final int packF(int acc, int aux, int nega) {
	int F = (nega << 6) | (((acc ^ aux) << 1) & F_H) | ((acc >> 4) & F_C);
	if ((acc & 0xFF) == 0)
	    F |= F_Z;
	return F;
    }

    private final int daa(int i) { // I
	boolean n = (i & 0x400) != 0;
	boolean h = (i & 0x200) != 0;
	boolean c = (i & 0x100) != 0;

	int nh = (i >> 4) & 0x0F;
	int nl = i & 0x0F;

	if (!n) {
	    int x = (c || nh >= 10 || (!h & nh == 9 && nl >= 10)) ? 6 : 0;
	    int y = (h || nl >= 10) ? 6 : 0;
	    return (x << 4) | y;
	}
	else {
	    switch ((i >> 8) & 3) {
	    default:
	    case 0: return 0;
	    case 1: return 0xA0;
	    case 2: return 0xFA;
	    case 3: return 0x9A;
	    }
	}
    }

    private final int daa_carry(int a) {
	return a >= 0x20 || a == 0x18 || a == 0x19 ? 0x10 : 0x00;
    }

    private final void cpu_state_save(DataOutputStream dos) throws IOException {
	dos.write(A);
	dos.write(F);
	dos.write(B);
	dos.write(C);
	dos.write(D);
	dos.write(E);
	dos.writeInt(HL);
	dos.writeInt(SP);
	dos.writeInt(PC);
	dos.writeBoolean(cpu_halt);
	dos.writeBoolean(cpu_ime);
	dos.writeBoolean(int_disable);
	dos.write(cpu_speed);
	//dos.writeInt(gdma_rest);
	dos.writeInt(rest_clock);
	dos.writeInt(sys_clock);
	dos.writeInt(div_clock);
	//dos.writeInt(total_clock);
    }

    private final void cpu_state_load(DataInputStream dis) throws IOException {
	A = dis.read() & 0xFF;
	F = dis.read() & 0xFF;
	B = dis.read() & 0xFF;
	C = dis.read() & 0xFF;
	D = dis.read() & 0xFF;
	E = dis.read() & 0xFF;
	HL = dis.readInt();
	SP = dis.readInt();
	PC = dis.readInt();
	cpu_halt = dis.readBoolean();
	cpu_ime = dis.readBoolean();
	int_disable = dis.readBoolean();
	cpu_speed = dis.read() & 0xFF;
	//gdma_rest = dis.readInt();
	rest_clock = dis.readInt();
	sys_clock = dis.readInt();
	div_clock = dis.readInt();
	//total_clock = dis.readInt();
    }


    // lcd
    private static final int LCDC_LCD_ON = 0x80;
    private static final int LCDC_WINDOW_TILE_MAP = 0x40;
    private static final int LCDC_WINDOW_ON = 0x20;
    private static final int LCDC_BG_TILE_DATA = 0x10;
    private static final int LCDC_BG_TILE_MAP = 0x08;
    private static final int LCDC_OBJ_SIZE = 0x04;
    private static final int LCDC_OBJ_ON = 0x02;
    private static final int LCDC_BG_ON = 0x01;

    private static final int STAT_LYCEQULY = 0x40;
    private static final int STAT_MODE10 = 0x20;
    private static final int STAT_VBLANK = 0x10;
    private static final int STAT_HBLANK = 0x08;
    private static final int STAT_COINCIDENCE = 0x04;
    private static final int STAT_MODE = 0x03;

    private static final int OBJ_PRIORITY = 0x80;
    private static final int OBJ_Y_FLIP = 0x40;
    private static final int OBJ_X_FLIP = 0x20;
    private static final int OBJ_PALETTE_BANK = 0x10;
    private static final int OBJ_TILE_BANK = 0x08;
    private static final int OBJ_PALETTE_INDEX = 0x07;

    private static final int[][][] dmg_pal = {
	{ // mN(Monochrome) -GB
	    { 0x7FFF, 0x56B5, 0x2D6B, 0x0, },
	    { 0x7FFF, 0x56B5, 0x2D6B, 0x0, },
	    { 0x7FFF, 0x56B5, 0x2D6B, 0x0, },
	},
	{ // 邢ZsA(Bright Sepia)
	    { 0x7FFF, 0x4F3F, 0x133, 0x0, },
	    { 0x7FFF, 0x4F3F, 0x133, 0x0, },
	    { 0x7FFF, 0x4F3F, 0x133, 0x0, },
	},
	{ // (Red)
	    { 0x7FFF, 0x673F, 0x18CF, 0x0, },
	    { 0x7FFF, 0x67F9, 0x19F6, 0x0, },
	    { 0x7FFF, 0x673F, 0x65F6, 0x0, },
	},
	{ // ÂZsA(Dark Sepia)
	    { 0x7FFF, 0x4ED9, 0x25F3, 0xC9, },
	    { 0x7FFF, 0x4F3F, 0x133, 0x0, },
	    { 0x7FFF, 0x4F3F, 0x133, 0x0, },
	},
	{ // pXeJ[(Pastel Color)
	    { 0x7FFF, 0x673F, 0x7F3F, 0x0, },
	    { 0x7FFF, 0x673F, 0x7F3F, 0x0, },
	    { 0x7FFF, 0x673F, 0x7F3F, 0x0, },
	},
	{ // IW(Orange)
	    { 0x7FFF, 0x333C, 0x18D9, 0x0, },
	    { 0x7FFF, 0x333C, 0x18D9, 0x0, },
	    { 0x7FFF, 0x333C, 0x18D9, 0x0, },
	},
	{ // (Yellow)
	    { 0x7FFF, 0x4F3F, 0xD3F, 0x0, },
	    { 0x7FFF, 0x673F, 0x65F6, 0x0, },
	    { 0x7FFF, 0x67F9, 0x19F6, 0x0, },
	},
	{ // 邢(Bright Blue)
	    { 0x7FFF, 0x673F, 0x65F6, 0x0, },
	    { 0x7FFF, 0x673F, 0x18CF, 0x0, },
	    { 0x7FFF, 0x67F9, 0x19F6, 0x0, },
	},
	{ // Â(Dark Blue)
	    { 0x7FFF, 0x7336, 0x3DFC, 0x0, },
	    { 0x7FFF, 0x673F, 0x18CF, 0x0, },
	    { 0x7FFF, 0x4F3F, 0x133, 0x0, },
	},
	{ // O[(Gray)
	    { 0x7FFF, 0x5B36, 0x2599, 0x0, },
	    { 0x7FFF, 0x5B36, 0x2599, 0x0, },
	    { 0x7FFF, 0x5B36, 0x2599, 0x0, },
	},
	{ // 邢(Bright Green)
	    { 0x7FFF, 0x3F9F, 0x65FF, 0x0, },
	    { 0x7FFF, 0x3F9F, 0x65FF, 0x0, },
	    { 0x7FFF, 0x3F9F, 0x65FF, 0x0, },
	},
	{ // Â(Dark Green) -Default
	    { 0x7FFF, 0x3F3F, 0x5A66, 0x0, },
	    { 0x7FFF, 0x4E79, 0x18CF, 0x0, },
	    { 0x7FFF, 0x4E79, 0x18CF, 0x0, },
	},
	{ // ](Reverse)
	    { 0x0, 0x5AC6, 0x4F3F, 0x7FFF, },
	    { 0x0, 0x5AC6, 0x4F3F, 0x7FFF, },
	    { 0x0, 0x5AC6, 0x4F3F, 0x7FFF, },
	},
	/*
	  { // SGBpbgϊp
	  { 0x0, 0x1, 0x2, 0x3, },
	  { 0x0, 0x1, 0x2, 0x3, },
	  { 0x0, 0x1, 0x2, 0x3, },
	  },
	*/
    };

    // save
    private byte[][] VRAM;
    private int[] OAM;
    private int[] palette_ram;
    // no save
    private byte[] scan_buf;
    private char[] patpix;
    private boolean[] patdirty;
    private boolean render_frame;
    private int now_win_line;
    private int n_mpal16;

    public final void xxx() {
	n_mpal16 = (n_mpal16 + 1) % dmg_pal.length;
	lcd_reset();
    }

    private final void lcd_init() {
	VRAM = new byte[4][0x1000];
	OAM = new int[0x100];
	palette_ram = new int[0x80];

	scan_buf = new byte[176];
	patpix = new char[1024 * 8];
	patdirty = new boolean[1024];

	now_win_line = 0;
    }

    private final void lcd_reset() {
	if (hw_type < HW_GBC) {
	    pal_write_dmg(0, 0, R_BGP);
	    //pal_write_dmg(8, 0, R_BGP);
	    pal_write_dmg(64, 1,R_OBP0);
	    pal_write_dmg(72, 2,R_OBP1);
	}

	for (int i = 0; i < patdirty.length; i++)
	    patdirty[i] = true;
    }

    private final void lcd_set_palette(int i, int data) {
	palette_ram[i] = data;
	lcd_update_palette(i >> 1);
    }

    private final void lcd_update_palette(int i) {
	int c = palette_ram[i << 1] | (palette_ram[(i << 1) + 1] << 8);
	int r = (c & 0x001F) << 3;
	int g = (c & 0x03E0) >> 2;
	int b = (c & 0x7C00) >> 7;
	r |= r >> 5;
	g |= g >> 5;
	b |= b >> 5;
	gui.setPalette(i, r, g, b);
    }

    private final void pal_write_dmg(int i, int mapnum, int d)
    {
        int j;
        int[] cmap = dmg_pal[n_mpal16][mapnum];
        int c, r, g, b;

        if (hw_type >= HW_GBC) return;


        for (j = 0; j < 8; j += 2) {
	    c = cmap[(d >> j) & 3];
	    //r = (c & 0xf8) >> 3;
	    //g = (c & 0xf800) >> 6;
	    //b = (c & 0xf80000) >> 9;
	    //c = r|g|b;
	    
	    lcd_set_palette(i+j, c & 0xff);
	    lcd_set_palette(i+j+1, c >> 8);
	}
    }

    private final void lcd_render_line() {
	if (!render_frame)
	    return;
	
	bg_render_line();
	wnd_render_line();
	obj_scan();

	gui.setPixels(R_LY, scan_buf, 7);
    }

    private final void bg_scan(int y, int LCDC_xx_TILE_MAP, int i, int n, int scx) {
	
	final byte[] tilemap = VRAM[1];
	final int base = ((R_LCDC & LCDC_xx_TILE_MAP) != 0 ? 0x0C00 : 0x0800) + ((y >> 3) << 5);
	final int tilemode = R_LCDC & LCDC_BG_TILE_DATA;

	final boolean[] patdirty = this.patdirty;
	final char[] patpix = this.patpix;
	final byte[] scan_buf = this.scan_buf;

	for (int j = 0; j < n; j++) {
	    final int x = (scx + j) & 0x1F;
	    int tile = 0x100 + tilemap[base + x];
	    if (tilemode != 0)
		tile &= 0xFF;

	    if (patdirty[tile])
		updatepatpix(tile);
	    int pat = patpix[(tile << 3) + (y & 7)];

	    scan_buf[i    ] = (byte)( pat        & 3);
	    scan_buf[i + 1] = (byte)((pat >>  2) & 3);
	    scan_buf[i + 2] = (byte)((pat >>  4) & 3);
	    scan_buf[i + 3] = (byte)((pat >>  6) & 3);
	    scan_buf[i + 4] = (byte)((pat >>  8) & 3);
	    scan_buf[i + 5] = (byte)((pat >> 10) & 3);
	    scan_buf[i + 6] = (byte)((pat >> 12) & 3);
	    scan_buf[i + 7] = (byte)((pat >> 14) & 3);
	    i += 8;
	}
    }

    private final void bg_scan_color(int y, int LCDC_xx_TILE_MAP, int i, int n, int scx) {
	final byte[] tilemap = VRAM[1];
	final byte[] attrmap = VRAM[3];
	final int base = ((R_LCDC & LCDC_xx_TILE_MAP) != 0 ? 0x0C00 : 0x0800) + ((y >> 3) << 5);
	final int tilemode = R_LCDC & LCDC_BG_TILE_DATA;

	final boolean[] patdirty = this.patdirty;
	final char[] patpix = this.patpix;
	final byte[] scan_buf = this.scan_buf;

	for (int j = 0; j < n; j++) {
	    final int x = (scx + j) & 0x1F;
	    final int attr = attrmap[base + x];
	    int tile = 0x100 + tilemap[base + x];
	    if (tilemode != 0)
		tile &= 0xFF;
	    tile |= (attr & OBJ_TILE_BANK) << 6;

	    if (patdirty[tile])
		updatepatpix(tile);
	    int pat = patpix[(tile << 3) + (((attr & OBJ_Y_FLIP) != 0 ? ~y : y) & 7)];
	    int colhi = ((attr & OBJ_PALETTE_INDEX) << 2) | (~attr & OBJ_PRIORITY);

	    if ((attr & OBJ_X_FLIP) != 0) {
		scan_buf[i    ] = (byte)(((pat >> 14) & 3) | colhi);
		scan_buf[i + 1] = (byte)(((pat >> 12) & 3) | colhi);
		scan_buf[i + 2] = (byte)(((pat >> 10) & 3) | colhi);
		scan_buf[i + 3] = (byte)(((pat >>  8) & 3) | colhi);
		scan_buf[i + 4] = (byte)(((pat >>  6) & 3) | colhi); 
		scan_buf[i + 5] = (byte)(((pat >>  4) & 3) | colhi);
		scan_buf[i + 6] = (byte)(((pat >>  2) & 3) | colhi);
		scan_buf[i + 7] = (byte)(( pat        & 3) | colhi);
	    }
	    else {
		scan_buf[i    ] = (byte)(( pat        & 3) | colhi);
		scan_buf[i + 1] = (byte)(((pat >>  2) & 3) | colhi);
		scan_buf[i + 2] = (byte)(((pat >>  4) & 3) | colhi);
		scan_buf[i + 3] = (byte)(((pat >>  6) & 3) | colhi);
		scan_buf[i + 4] = (byte)(((pat >>  8) & 3) | colhi); 
		scan_buf[i + 5] = (byte)(((pat >> 10) & 3) | colhi);
		scan_buf[i + 6] = (byte)(((pat >> 12) & 3) | colhi);
		scan_buf[i + 7] = (byte)(((pat >> 14) & 3) | colhi);
	    }
	    
	    i += 8;
	}
    }

    private final void bg_render_line() {

	if ((R_LCDC & LCDC_BG_ON) == 0) {
	    for (int i = 0; i < 176; i++)
		scan_buf[i] = 0;
	    return;
	}

	if (hw_type >= HW_GBC)
	    bg_scan_color((R_SCY + R_LY) & 0xFF, LCDC_BG_TILE_MAP, ~R_SCX & 7, 21, R_SCX >> 3);
	else
	    bg_scan((R_SCY + R_LY) & 0xFF, LCDC_BG_TILE_MAP, ~R_SCX & 7, 21, R_SCX >> 3);

    }

    private final void wnd_render_line() {
	if ((R_LCDC & LCDC_WINDOW_ON) == 0 || R_WX < 0 || R_WX >= 167 || R_WY < 0 || R_WY > R_LY)
	    return;

	if (hw_type >= HW_GBC)
	    bg_scan_color(now_win_line++, LCDC_WINDOW_TILE_MAP, R_WX, 21 - (R_WX >> 3), 0);
	else
	    bg_scan(now_win_line++, LCDC_WINDOW_TILE_MAP, R_WX, 21 - (R_WX >> 3), 0);
    }

    private final void obj_scan() {
		if ((R_LCDC & LCDC_OBJ_ON) == 0)
			return;

		final int[] OAM = this.OAM;
		final int R_LY = this.R_LY;

		final boolean[] patdirty = this.patdirty;
		final char[] patpix = this.patpix;
		final byte[] scan_buf = this.scan_buf;

		final boolean spr8x16 = (R_LCDC & LCDC_OBJ_SIZE) != 0;

		for (int i = 0; i < 40; i++) {
		    final int s = i << 2;

		    final int y = OAM[s];
		    final int x = OAM[s + 1];
		    int tile = OAM[s + 2];
		    final int flags = OAM[s + 3];

		    int v = R_LY + 16 - y;
		    if (spr8x16) {
				if (y <= R_LY || y > R_LY + 16 || x <= 0 || x > 167)
					continue;

					tile &= ~1;
					if ((flags & OBJ_Y_FLIP) != 0)
					    v = 15 - v;
	    	} else {
				if (y <= R_LY + 8 || y > R_LY + 16 || x <= 0 || x > 167)
				    continue;

				if ((flags & OBJ_Y_FLIP) != 0)
					v = 7 - v;
		    }

		    final int colhi;
			if (hw_type >= HW_GBC) {
				tile |= (flags & OBJ_TILE_BANK) << 6;
				colhi = 0x20 + ((flags & OBJ_PALETTE_INDEX) << 2);
			} else {
				colhi = 0x20 + ((flags & OBJ_PALETTE_BANK) >> 2);
			}

		    final int tilev = (tile << 3) + v;
		    tile = tilev >> 3;
		    if (patdirty[tile])
			updatepatpix(tile);
		    int pat = patpix[tilev];

		    final int j = x - 1;
		    int c;
		    // XvCg̗D揇ʂނႭBBB
		    if ((flags & OBJ_PRIORITY) != 0)
			if ((flags & OBJ_X_FLIP) != 0) {
			    if ((scan_buf[j    ] & 3) == 0 && (c = (pat >> 14) & 3) != 0) scan_buf[j    ] = (byte)(colhi | c);
			    if ((scan_buf[j + 1] & 3) == 0 && (c = (pat >> 12) & 3) != 0) scan_buf[j + 1] = (byte)(colhi | c);
			    if ((scan_buf[j + 2] & 3) == 0 && (c = (pat >> 10) & 3) != 0) scan_buf[j + 2] = (byte)(colhi | c);
			    if ((scan_buf[j + 3] & 3) == 0 && (c = (pat >>  8) & 3) != 0) scan_buf[j + 3] = (byte)(colhi | c);
			    if ((scan_buf[j + 4] & 3) == 0 && (c = (pat >>  6) & 3) != 0) scan_buf[j + 4] = (byte)(colhi | c);
			    if ((scan_buf[j + 5] & 3) == 0 && (c = (pat >>  4) & 3) != 0) scan_buf[j + 5] = (byte)(colhi | c);
			    if ((scan_buf[j + 6] & 3) == 0 && (c = (pat >>  2) & 3) != 0) scan_buf[j + 6] = (byte)(colhi | c);
			    if ((scan_buf[j + 7] & 3) == 0 && (c =  pat        & 3) != 0) scan_buf[j + 7] = (byte)(colhi | c);
			} else {
			    if ((scan_buf[j    ] & 3) == 0 && (c =  pat        & 3) != 0) scan_buf[j    ] = (byte)(colhi | c);
			    if ((scan_buf[j + 1] & 3) == 0 && (c = (pat >>  2) & 3) != 0) scan_buf[j + 1] = (byte)(colhi | c);
			    if ((scan_buf[j + 2] & 3) == 0 && (c = (pat >>  4) & 3) != 0) scan_buf[j + 2] = (byte)(colhi | c);
			    if ((scan_buf[j + 3] & 3) == 0 && (c = (pat >>  6) & 3) != 0) scan_buf[j + 3] = (byte)(colhi | c);
			    if ((scan_buf[j + 4] & 3) == 0 && (c = (pat >>  8) & 3) != 0) scan_buf[j + 4] = (byte)(colhi | c);
			    if ((scan_buf[j + 5] & 3) == 0 && (c = (pat >> 10) & 3) != 0) scan_buf[j + 5] = (byte)(colhi | c);
			    if ((scan_buf[j + 6] & 3) == 0 && (c = (pat >> 12) & 3) != 0) scan_buf[j + 6] = (byte)(colhi | c);
			    if ((scan_buf[j + 7] & 3) == 0 && (c = (pat >> 14) & 3) != 0) scan_buf[j + 7] = (byte)(colhi | c);
			} else if (hw_type >= HW_GBC)
			if ((flags & OBJ_X_FLIP) != 0) {
				    if ((scan_buf[j    ] & ~0x1C) <= 0 && (c = (pat >> 14) & 3) != 0) scan_buf[j    ] = (byte)(colhi | c);
				    if ((scan_buf[j + 1] & ~0x1C) <= 0 && (c = (pat >> 12) & 3) != 0) scan_buf[j + 1] = (byte)(colhi | c);
				    if ((scan_buf[j + 2] & ~0x1C) <= 0 && (c = (pat >> 10) & 3) != 0) scan_buf[j + 2] = (byte)(colhi | c);
				    if ((scan_buf[j + 3] & ~0x1C) <= 0 && (c = (pat >>  8) & 3) != 0) scan_buf[j + 3] = (byte)(colhi | c);
				    if ((scan_buf[j + 4] & ~0x1C) <= 0 && (c = (pat >>  6) & 3) != 0) scan_buf[j + 4] = (byte)(colhi | c);
				    if ((scan_buf[j + 5] & ~0x1C) <= 0 && (c = (pat >>  4) & 3) != 0) scan_buf[j + 5] = (byte)(colhi | c);
				    if ((scan_buf[j + 6] & ~0x1C) <= 0 && (c = (pat >>  2) & 3) != 0) scan_buf[j + 6] = (byte)(colhi | c);
				    if ((scan_buf[j + 7] & ~0x1C) <= 0 && (c = (pat >>  0) & 3) != 0) scan_buf[j + 7] = (byte)(colhi | c);
			} else {
			    if ((scan_buf[j    ] & ~0x1C) <= 0 && (c = (pat >>  0) & 3) != 0) scan_buf[j    ] = (byte)(colhi | c);
			    if ((scan_buf[j + 1] & ~0x1C) <= 0 && (c = (pat >>  2) & 3) != 0) scan_buf[j + 1] = (byte)(colhi | c);
			    if ((scan_buf[j + 2] & ~0x1C) <= 0 && (c = (pat >>  4) & 3) != 0) scan_buf[j + 2] = (byte)(colhi | c);
			    if ((scan_buf[j + 3] & ~0x1C) <= 0 && (c = (pat >>  6) & 3) != 0) scan_buf[j + 3] = (byte)(colhi | c);
			    if ((scan_buf[j + 4] & ~0x1C) <= 0 && (c = (pat >>  8) & 3) != 0) scan_buf[j + 4] = (byte)(colhi | c);
			    if ((scan_buf[j + 5] & ~0x1C) <= 0 && (c = (pat >> 10) & 3) != 0) scan_buf[j + 5] = (byte)(colhi | c);
			    if ((scan_buf[j + 6] & ~0x1C) <= 0 && (c = (pat >> 12) & 3) != 0) scan_buf[j + 6] = (byte)(colhi | c);
			    if ((scan_buf[j + 7] & ~0x1C) <= 0 && (c = (pat >> 14) & 3) != 0) scan_buf[j + 7] = (byte)(colhi | c);
			} else if ((flags & OBJ_X_FLIP) != 0) {
			    if ((c = (pat >> 14) & 3) != 0) scan_buf[j    ] = (byte)(colhi | c);
			    if ((c = (pat >> 12) & 3) != 0) scan_buf[j + 1] = (byte)(colhi | c);
			    if ((c = (pat >> 10) & 3) != 0) scan_buf[j + 2] = (byte)(colhi | c);
			    if ((c = (pat >>  8) & 3) != 0) scan_buf[j + 3] = (byte)(colhi | c);
			    if ((c = (pat >>  6) & 3) != 0) scan_buf[j + 4] = (byte)(colhi | c);
			    if ((c = (pat >>  4) & 3) != 0) scan_buf[j + 5] = (byte)(colhi | c);
			    if ((c = (pat >>  2) & 3) != 0) scan_buf[j + 6] = (byte)(colhi | c);
			    if ((c =  pat        & 3) != 0) scan_buf[j + 7] = (byte)(colhi | c);
			} else {
			    if ((c =  pat        & 3) != 0) scan_buf[j    ] = (byte)(colhi | c);
			    if ((c = (pat >>  2) & 3) != 0) scan_buf[j + 1] = (byte)(colhi | c);
			    if ((c = (pat >>  4) & 3) != 0) scan_buf[j + 2] = (byte)(colhi | c);
			    if ((c = (pat >>  6) & 3) != 0) scan_buf[j + 3] = (byte)(colhi | c);
			    if ((c = (pat >>  8) & 3) != 0) scan_buf[j + 4] = (byte)(colhi | c);
			    if ((c = (pat >> 10) & 3) != 0) scan_buf[j + 5] = (byte)(colhi | c);
			    if ((c = (pat >> 12) & 3) != 0) scan_buf[j + 6] = (byte)(colhi | c);
			    if ((c = (pat >> 14) & 3) != 0) scan_buf[j + 7] = (byte)(colhi | c);
			}
		}
    }

    private final void updatepatpix(int patno) {
		
		final byte[] vram = VRAM[(patno >> 8) & 3];
		int a = (patno & 0xFF) << 4;

		final char[] patpix = this.patpix;
	
		for (int i = 0; i < 8; i++, a += 2) {
		    int lo = vram[a];
		    int hi = vram[a + 1] << 1;
		    int pat = 0;

		    for (int j = 0; j < 8; j++) {
				pat = (pat << 2) | (lo & 1) | (hi & 2);
				lo >>= 1;
				hi >>= 1;
		    }
		    patpix[(patno << 3) + i] = (char)pat;
		}
	
		patdirty[patno] = false;
    }

    private final void lcd_state_save(DataOutputStream dos) throws IOException {
	for (int i = 0; i < VRAM.length; i++)
	    dos.write(VRAM[i]);
	for (int i = 0; i < OAM.length; i++)
	    dos.write(OAM[i]);
	for (int i = 0; i < palette_ram.length; i++)
	    dos.write(palette_ram[i]);
    }

    private final void lcd_state_load(DataInputStream dis) throws IOException {
	for (int i = 0; i < VRAM.length; i++)
	    dis.read(VRAM[i]);
	for (int i = 0; i < OAM.length; i++)
	    OAM[i] = dis.read() & 0xFF;
	for (int i = 0; i < palette_ram.length; i++)
	    palette_ram[i] = dis.read() & 0xFF;

	for (int i = 0; i < 0x40; i++)
	    lcd_update_palette(i);

	for (int i = 0; i < patdirty.length; i++)
	    patdirty[i] = true;
    }


    // mbc
    // save
    private byte[][] SRAM;
    private int MR_RAMG; // RAM/Clock write protect
    private int MR_ROMB; // ROM Bank Select
    private int MR_RAMB; // RAM Bank/Clock Select
    private boolean mbc_model;
    // nosave
    private byte[][] ROM;
    private int cart_type;

    private final void mbc_init() {
    }

    private final void mbc_reset() {
	MR_RAMG = 0;
	MR_ROMB = 1;
	MR_RAMB = 0;
	mbc_model = false;
    }

    private final void mbc_write(int addr, int data) {
	switch (cart_type) {
	case 0: // ROM only
	case 8: // ROM+RAM
	case 9: // ROM+RAM+BATT
	    return;
	case 1: // MBC1
	case 2: // MBC1+RAM
	case 3: // MBC1+RAM+BATT
	    mbc1_write(addr, data);
	    break;
	case 5: // MBC2
	case 6: // MBC2+BATT
	    mbc2_write(addr, data);
	    break;
	case 0x0B: // MMM01
	case 0x0C: // MMM01+SRAM
	case 0x0D: // MMM01+SRAM+BATT
	    mmm01_write(addr, data);
	    break;
	case 0x0F: // MBC3+TIMER+BATT
	case 0x10: // MBC3+TIMER+RAM+BATT
	case 0x11: // MBC3
	case 0x12: // MBC3+RAM
	case 0x13: // MBC3+RAM+BATT
	    mbc3_write(addr, data);
	    break;
	case 0x19: // MBC5
	case 0x1A: // MBC5+RAM
	case 0x1B: // MBC5+RAM+BATT
	case 0x1C: // MBC5+RUMBLE
	case 0x1D: // MBC5+RUMBLE+SRAM
	case 0x1E: // MBC5+RUMBLE+SRAM+BATT
	    mbc5_write(addr, data);
	    break;
	    /*
	      case 0x22: // RRJ[rBB
	      mbc7_write(addr, data);
	      break;
	    */
	case 0xFD: // Bandai TAMA5
	    tama5_write(addr, data);

	    // Ⴂ킩
	case 0xFE: // Hudson HuC-3
	    mbc3_write(addr, data);
	    break;
	case 0xFF: // Hudson HuC-1
	    huc1_write(addr, data);
	    break;

	default:
	    throw new Error("Unknown cartridge type " + Integer.toHexString(cart_type));
	}

	mem_set_ROM_bank();
    }

    public final void mbc1_write(int addr, int data) {
	switch (addr >> 12) {
	case 0:
	case 1:
	    MR_RAMG = data & 0x0F;
	    break;
	case 2:
	case 3:
	    if ((data &= 0x1F) == 0)
		data = 0x01;
	    MR_ROMB = (MR_ROMB & 0x60) | data;
	    break;
	case 4:
	case 5:
	    data &= 3;
	    if (mbc_model)
		MR_RAMB = data;
	    else
		MR_ROMB = (MR_ROMB & 0x1F) | (data << 5);
	    break;
	case 6:
	case 7:
	    mbc_model = (data & 1) != 0;
	    break;
	}
    }

    public final void mbc2_write(int addr, int data) {
	if ((addr & 0x0100) == 0)
	    MR_RAMG = data & 0x0F;
	if ((addr & 0xE100) == 0x2100)
	    MR_ROMB = data & 0x0F;
    }

    public final void mbc3_write(int addr, int data) {
	switch (addr >> 12) {
	case 0:
	case 1:
	    MR_RAMG = data & 0x0F;
	    break;
	case 2:
	case 3:
	    if ((data &= 0x7F) == 0)
		data = 0x01;
	    MR_ROMB = data;
	    break;
	case 4:
	case 5:
	    MR_RAMB = data & 3;
	    rtc_sel = data & 0x0F;
	    break;
	case 6:
	case 7:
	    rtc_latch(data);
	    break;
	}
    }

    public final void mbc5_write(int addr, int data) {
	switch (addr >> 12) {
	case 0:
	case 1:
	    MR_RAMG = data & 0x0F;
	    break;
	case 2:
	    MR_ROMB = (MR_ROMB & 0x0100) | data;
	    break;
	case 3:
	    MR_ROMB = (MR_ROMB & 0x00FF) | ((data & 1) << 8);
	    break;
	case 4:
	case 5:
	    MR_RAMB = data & 0x0F;
	    if (cart_type >= 0x1C && cart_type <= 0x1E) // Rumble
		gui.setVibration((data & 8) != 0);
	    break;
	}
    }

    public final void huc1_write(int addr, int data) {
	switch (addr >> 12) {
	case 0:
	case 1:
	    MR_RAMG = data & 0x0F;
	    break;
	case 2:
	case 3:
	    if ((data &= 0x3F) == 0) // MBC1Ƃ̈Ⴂ͂H
		data = 0x01;
	    MR_ROMB = (MR_ROMB & 0x60) | data;
	    break;
	case 4:
	case 5:
	    data &= 3;
	    if (mbc_model)
		MR_RAMB = data;
	    else
		MR_ROMB = (MR_ROMB & 0x3F) | (data << 5);
	    break;
	case 6:
	case 7:
	    mbc_model = (data & 1) != 0;
	    break;
	}
    }

    public final void tama5_write(int addr, int data) {
	// Ȃ
    }

    public final void mmm01_write(int addr, int data) {
	throw new Error("MMM01 is not supporeted");
    }

    public final void mbc_state_save(DataOutputStream dos) throws IOException {
	// SRAM
	dos.write(MR_RAMG);
	dos.writeInt(MR_ROMB);
	dos.write(MR_RAMB);
	dos.writeBoolean(mbc_model);
    }

    public final void mbc_state_load(DataInputStream dis) throws IOException {
	// SRAM
	MR_RAMG = dis.read() & 0xFF;
	MR_ROMB = dis.readInt();
	MR_RAMB = dis.read() & 0xFF;
	mbc_model = dis.readBoolean();
    }


    // rtc
    private static final int[] days_of_month = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
    private static final int RTC_CARRY = 0x80;
    private static final int RTC_STOP = 0x40;
    // no save
    private Calendar calendar;
    // save
    private int[] rtc_regs;
    private int rtc_sel;
    private int rtc_latch;

    private final void rtc_init() {
	calendar = Calendar.getInstance();
	rtc_regs = new int[8];
    }

    private final void rtc_reset() {
	rtc_sel = 0;
	rtc_latch = 0;
    }

    private final void rtc_latch(int data) {
	if ((~rtc_latch & data & 1) != 0) {
	    calendar.setTime(new Date(System.currentTimeMillis()));
	    int olddoy = rtc_regs[3] | ((rtc_regs[4] & 1) << 8);
	    int doy = calendar.get(Calendar.DAY_OF_MONTH);
	    for (int i = 0; i < calendar.get(Calendar.MONTH); i++)
		doy += days_of_month[i];
	    rtc_regs[0] = calendar.get(Calendar.SECOND);
	    rtc_regs[1] = calendar.get(Calendar.MINUTE);
	    rtc_regs[2] = calendar.get(Calendar.HOUR_OF_DAY);
	    rtc_regs[3] = doy & 0xFF;
	    rtc_regs[4] = (doy < olddoy ? RTC_CARRY : 0) | (rtc_regs[4] & RTC_STOP) | (doy >> 8);
	    rtc_regs[5] = 0xFF;
	    rtc_regs[6] = 0xFF;
	    rtc_regs[7] = 0xFF;
	}
	rtc_latch = data;
    }

    private final int rtc_read() {
	return rtc_regs[rtc_sel & 7];
    }

    private final void rtc_write(int data) {
	if ((rtc_sel & 8) == 0)
	    return;
	switch (rtc_sel & 7) {
	case 4:
	    rtc_regs[4] = (rtc_regs[4] & 1) | (data & (RTC_CARRY|RTC_STOP));
	    break;
	}
    }

    private final void rtc_state_save(DataOutputStream dos) throws IOException {
	for (int i = 0; i < rtc_regs.length; i++)
	    dos.write(rtc_regs[i]);
	dos.write(rtc_sel);
	dos.write(rtc_latch);
    }

    private final void rtc_state_load(DataInputStream dis) throws IOException {
	for (int i = 0; i < rtc_regs.length; i++)
	    rtc_regs[i] = dis.read() & 0xFF;
	rtc_sel = dis.read() & 0xFF;
	rtc_latch = dis.read() & 0xFF;
    }

    
    // pad
    public static final int PAD_RIGHT = 0;
    public static final int PAD_LEFT = 1;
    public static final int PAD_UP = 2;
    public static final int PAD_DOWN = 3;
    public static final int PAD_A = 4;
    public static final int PAD_B = 5;
    public static final int PAD_SELECT = 6;
    public static final int PAD_START = 7;
    
    private final void pad_init() {
    }
    
    private final void pad_reset() {
	gui.resetJoyPad();
    }

    private final void pad_refresh() {
	int oldP1 = R_P1;

	R_P1 = (R_P1 & 0x30) | 0xC0;
	int pad_state = gui.readJoyPad();
	if ((R_P1 & 0x10) == 0)
	    R_P1 |= pad_state & 0x0F;
	if ((R_P1 & 0x20) == 0)
	    R_P1 |= (pad_state >> 4) & 0x0F;
	R_P1 ^= 0x0F;

	if (((oldP1 & ~R_P1) & 0x0F) != 0)
	    R_IF |= INT_PAD;
    }

    private final void pad_state_save(DataOutputStream dos) throws IOException {
    }

    private final void pad_state_load(DataInputStream dis) throws IOException {
    }

    
    // emu
    // no save
    private String rom_name;
    private int request;
    private int param;

    public final void init() {
	hw_init();
	cpu_init();
	mem_init();
	lcd_init();
	rtc_init();
	pad_init();
	mbc_init();
    }

    public final void reset() {
	hw_reset();
	cpu_reset();
	lcd_reset();
	rtc_reset();
	pad_reset();
	mbc_reset();
	mem_warmreset();
    }

    public final void do_line() {
	if ((R_LCDC & LCDC_LCD_ON) != 0) {
	    R_LY = (R_LY + 1) % 154;

	    R_STAT &= ~(STAT_COINCIDENCE|STAT_MODE);
	    
	    if (R_LY == R_LYC) {
		R_STAT |= STAT_COINCIDENCE;
		if ((R_STAT & STAT_LYCEQULY) != 0)
		    R_IF |= INT_LCDC;
	    }

	    if (R_LY == 0) {
		render_frame = gui.startFrame();
		now_win_line = 0;
	    }
	    
	    if (R_LY >= 144) { // VBlank period
		R_STAT |= 1; // Mode01 (V-Blank period)
		if (R_LY == 144) {
		    gui.endFrame(render_frame);

		    cpu_exec(72); // TGBdualł72ɂȂĂ̂͊荞݂8TCN̂lĂH̉ӏł͍lĂȂ悤B
		    R_IF |= INT_VBLANK;
		    if ((R_STAT & STAT_VBLANK) != 0)
			R_IF |= INT_LCDC;
		    cpu_exec(376);
		}
		else if (R_LY == 153) {
		    cpu_exec(80);
		    R_LY = 0;
		    cpu_exec(376); // ÕĈȂ葁ڂ0ɂȂ悤B
		    R_LY = 153;
		}
		else { // 80NbNڂLY邪蔲B
		    cpu_exec(456);
		}
	    }
	    else {
		R_STAT |= 2; // Mode10 (OAM busy)
		if ((R_STAT & STAT_MODE10) != 0)
		    R_IF |= INT_LCDC;
		cpu_exec(80);
		
		R_STAT |= 3; // Mode11 (OAM and VRAM busy)
		cpu_exec(169);
		
		if ((R_HDMA5 & 0x80) == 0) {
		    hw_hdma();
		    R_STAT &= ~STAT_MODE;
		    lcd_render_line();
		    cpu_exec(207);
		}
		else {
		    R_STAT &= ~STAT_MODE; // Mode00
		    lcd_render_line();
		    if ((R_STAT & STAT_HBLANK) != 0)
			R_IF |= INT_LCDC;
		    cpu_exec(207);
		}
	    }
	}
	else {
	    R_LY = 0;
	    now_win_line = 0;
	    R_STAT &= ~(STAT_COINCIDENCE|STAT_MODE);
	    cpu_exec(456);
	}
    }

    public final void run() {
	for (;;) {
	    switch (request) {
	    case GBCanvas.REQUEST_RESET:
		reset();
		break;
	    case GBCanvas.REQUEST_STOP:
		return;
	    case GBCanvas.REQUEST_LOAD:
		try {
		    state_load(gui.getDataInputStream(param));
		}
		catch (IOException e) {
		    System.err.println("[hɎs܂: " + e);
		}
		break;
	    case GBCanvas.REQUEST_SAVE:
		try {
		    state_save(gui.getDataOutputStream(param));
		}
		catch (IOException e) {
		    System.err.println("Z[uɎs܂: " + e);
		}
		break;
	    }
	    request = 0;

	    do_line();
	}
    }

    public final int loadROM(InputStream is) throws IOException {
	byte[] buf = load_4K_page(is);

	StringBuffer sb = new StringBuffer();
	int c;
	for (int i = 0; i < 0x0F; i++) {
	    if ((c = buf[0x134 + i] & 0xFF) == 0)
		break;
	    sb.append((char)c);
	}
	rom_name = sb.toString();

	hw_type = (buf[0x0143] & 0x80) != 0 ? 3 : buf[0x0146] == 0x03 ? 2 : 1;

	cart_type = buf[0x0147] & 0xFF;

	int n = buf[0x0148];
	int rom_banks = n == 0x52 ? 72 : n == 0x53 ? 80 : n == 0x54 ? 96 : 2 << n;
	ROM = new byte[rom_banks << 2][0x1000];
	ROM[0] = buf;
	for (int i = 1; i < ROM.length; i++) {
	    ROM[i] = load_4K_page(is);
	    gui.setMessage(((i + 1) << 2) + "KBǂݍ݂܂");
	}

	int sram_banks = 0;
	switch (buf[0x0149]) { // ͂ǂꂪH
	default:
	case 0:
	case 1:
	case 2: sram_banks = 1; break;
	case 3: sram_banks = 4; break;
	case 4:
	case 6: sram_banks = 16; break;
	case 5: sram_banks = 8; break;
	}

	SRAM = new byte[sram_banks << 1][0x1000];
	// loadSRAM();

	System.out.println("hw_type=" + hw_type);
	System.out.println("cart_type=0x" + Integer.toHexString(cart_type));
	System.out.println("ROM=" + ROM.length * 0x1000);
	System.out.println("SRAM=" + SRAM.length * 0x1000);

	return 0;
    }

    private final byte[] load_4K_page(InputStream is) throws IOException {
	byte[] buf = new byte[4096];
	int p = 0, n;

	while (p < 4096) {
	    if ((n = is.read(buf, p, 4096 - p)) < 0)
		throw new IOException("Unexpected EOF");
	    p += n;
	}
	return buf;
    }

    public final void request(int _request, int _param) {
	request = _request;
	param = _param;
    }

    private final void state_save(DataOutputStream dos) throws IOException {
	hw_state_save(dos);
	dos.writeInt(0xDEADBEEF);
	cpu_state_save(dos);
	dos.writeInt(0xDEADBEEF);
	lcd_state_save(dos);
	dos.writeInt(0xDEADBEEF);
	mbc_state_save(dos);
	dos.writeInt(0xDEADBEEF);
	rtc_state_save(dos);
	dos.writeInt(0xDEADBEEF);
	pad_state_save(dos);
	dos.writeInt(0xDEADBEEF);

	// SRAM͂ǂۑ悤H
	for (int i = 0; i < SRAM.length; i++)
	    dos.write(SRAM[i]);
    }

    private final void state_load(DataInputStream dis) throws IOException {
	hw_state_load(dis);
	if (dis.readInt() != 0xDEADBEEF) throw new IOException("hw");
	cpu_state_load(dis);
	if (dis.readInt() != 0xDEADBEEF) throw new IOException("cpu");
	lcd_state_load(dis);
	if (dis.readInt() != 0xDEADBEEF) throw new IOException("lcd");
	mbc_state_load(dis);
	if (dis.readInt() != 0xDEADBEEF) throw new IOException("mbc");
	rtc_state_load(dis);
	if (dis.readInt() != 0xDEADBEEF) throw new IOException("rtc");
	pad_state_load(dis);
	if (dis.readInt() != 0xDEADBEEF) throw new IOException("pad");
	//
	for (int i = 0; i < SRAM.length; i++)
	    dis.read(SRAM[i]);

	mem_warmreset();
    }
}
